@@ -1,7 +1,7 @@
 /*
  * %CopyrightBegin%
  *
- * Copyright Ericsson AB 2020-2022. All Rights Reserved.
+ * Copyright Ericsson AB 2020-2024. All Rights Reserved.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -453,8 +453,8 @@ static int parse_line_chunk(BeamFile *beam, IFF_Chunk *chunk) {
      * have to special-case it anywhere else. */
     name_count++;
 
-    /* Flags are unused at the moment. */
-    (void)flags;
+    /* Save flags. */
+    lines->flags = flags;
 
     /* Reserve space for the "undefined location" entry. */
     item_count++;
@@ -527,7 +527,7 @@ static int parse_line_chunk(BeamFile *beam, IFF_Chunk *chunk) {
         hp = name_heap;
         name = erts_atom_to_string(&hp, beam->module, suffix);
 
-        lines->names[0] = beamfile_add_literal(beam, name);
+        lines->names[0] = beamfile_add_literal(beam, name, 1);
 
         for (i = 1; i < name_count; i++) {
             Uint num_chars, num_built, num_eaten;
@@ -563,7 +563,7 @@ static int parse_line_chunk(BeamFile *beam, IFF_Chunk *chunk) {
                 ASSERT(num_built == num_chars);
                 ASSERT(num_eaten == name_length);
 
-                lines->names[i] = beamfile_add_literal(beam, name);
+                lines->names[i] = beamfile_add_literal(beam, name, 1);
 
                 if (name_heap != default_name_buf) {
                     erts_free(ERTS_ALC_T_LOADER_TMP, name_heap);
@@ -595,30 +595,22 @@ static void init_fallback_type_table(BeamFile *beam) {
     types->fallback = 1;
 
     types->entries[0].type_union = BEAM_TYPE_ANY;
-    types->entries[0].min = 0;
-    types->entries[0].max = -1;
+    types->entries[0].metadata_flags = 0;
+    types->entries[0].size_unit = 1;
+    types->entries[0].min = MAX_SMALL + 1;
+    types->entries[0].max = MIN_SMALL - 1;
 }
 
-static int parse_type_chunk(BeamFile *beam, IFF_Chunk *chunk) {
+static int parse_type_chunk_data(BeamFile *beam, BeamReader *p_reader) {
     BeamFile_TypeTable *types;
-    BeamReader reader;
 
-    Sint32 version, count;
+    Sint32 count;
     int i;
 
-    beamreader_init(chunk->data, chunk->size, &reader);
-
-    LoadAssert(beamreader_read_i32(&reader, &version));
-    if (version != BEAM_TYPES_VERSION) {
-        /* Incompatible type format. */
-        init_fallback_type_table(beam);
-        return 1;
-    }
-
     types = &beam->types;
     ASSERT(types->entries == NULL);
 
-    LoadAssert(beamreader_read_i32(&reader, &count));
+    LoadAssert(beamreader_read_i32(p_reader, &count));
     LoadAssert(CHECK_ITEM_COUNT(count, 0, sizeof(types->entries[0])));
     LoadAssert(count >= 1);
 
@@ -629,9 +621,13 @@ static int parse_type_chunk(BeamFile *beam, IFF_Chunk *chunk) {
 
     for (i = 0; i < count; i++) {
         const byte *type_data;
+        int extra;
 
-        LoadAssert(beamreader_read_bytes(&reader, 18, &type_data));
-        LoadAssert(beam_types_decode(type_data, 18, &types->entries[i]));
+        LoadAssert(beamreader_read_bytes(p_reader, 2, &type_data));
+        extra = beam_types_decode_type(type_data, &types->entries[i]);
+        LoadAssert(extra >= 0);
+        LoadAssert(beamreader_read_bytes(p_reader, extra, &type_data));
+        beam_types_decode_extra(type_data, &types->entries[i]);
     }
 
     /* The first entry MUST be the "any type." */
@@ -641,6 +637,23 @@ static int parse_type_chunk(BeamFile *beam, IFF_Chunk *chunk) {
     return 1;
 }
 
+static int parse_type_chunk(BeamFile *beam, IFF_Chunk *chunk) {
+    BeamReader reader;
+    Sint32 version;
+
+    beamreader_init(chunk->data, chunk->size, &reader);
+
+    LoadAssert(beamreader_read_i32(&reader, &version));
+
+    if (version == BEAM_TYPES_VERSION) {
+        return parse_type_chunk_data(beam, &reader);
+    } else {
+        /* Incompatible type format. */
+        init_fallback_type_table(beam);
+        return 1;
+    }
+}
+
 static ErlHeapFragment *new_literal_fragment(Uint size)
 {
     ErlHeapFragment *bp;
@@ -712,6 +725,11 @@ static int parse_decompressed_literals(BeamFile *beam,
         ErlHeapFragment *fragments;
         Eterm value;
 
+#ifdef DEBUG
+        erts_literal_area_t purge_area;
+        INITIALIZE_LITERAL_PURGE_AREA(purge_area);
+#endif
+
         LoadAssert(beamreader_read_i32(&reader, &ext_size));
         LoadAssert(beamreader_read_bytes(&reader, ext_size, &ext_data));
         term_size = erts_decode_ext_size(ext_data, ext_size);
@@ -727,14 +745,24 @@ static int parse_decompressed_literals(BeamFile *beam,
             erts_factory_close(&factory);
 
             LoadAssert(!is_non_value(value));
+            ASSERT(size_object_litopt(value, &purge_area) > 0);
 
             heap_size += erts_used_frag_sz(factory.heap_frags);
             fragments = factory.heap_frags;
         } else {
             erts_factory_dummy_init(&factory);
             value = erts_decode_ext(&factory, &ext_data, 0);
+
+            /* erts_decode_ext may return terms that are (or contain) global
+             * literals, for instance export funs or the empty tuple. As these
+             * are singleton values that belong to everyone, they can safely be
+             * returned without being copied into a fragment.
+             *
+             * (Note that erts_decode_ext_size does not include said term in
+             * the decoded size) */
             LoadAssert(!is_non_value(value));
-            ASSERT(is_immed(value));
+            ASSERT(size_object_litopt(value, &purge_area) == 0);
+
             fragments = NULL;
         }
 
@@ -1134,7 +1162,7 @@ void beamfile_free(BeamFile *beam) {
     }
 }
 
-Sint beamfile_add_literal(BeamFile *beam, Eterm term) {
+Sint beamfile_add_literal(BeamFile *beam, Eterm term, int deduplicate) {
     BeamFile_LiteralTable *literals;
     BeamFile_LiteralEntry *entries;
 
@@ -1146,22 +1174,17 @@ Sint beamfile_add_literal(BeamFile *beam, Eterm term) {
     literals = &beam->dynamic_literals;
     entries = literals->entries;
 
-    if (entries == NULL) {
-        literals->allocated = 32;
-
-        entries = erts_alloc(ERTS_ALC_T_PREPARED_CODE,
-                             literals->allocated * sizeof(*entries));
-
-        literals->entries = entries;
-    } else {
-        /* Return a matching index if this literal already exists. We search
-         * backwards since duplicates tend to be used close to one another,
-         * and skip searching static literals as the chances of overlap are
-         * pretty slim. */
-        for (i = literals->count - 1; i >= 0; i--) {
-            if (EQ(term, entries[i].value)) {
-                /* Dynamic literal indexes are negative, starting at -1 */
-                return ~i;
+    if (entries != NULL) {
+        if (deduplicate) {
+            /* Return a matching index if this literal already exists.
+             * We search backwards since duplicates tend to be used close to
+             * one another, and skip searching static literals as the chances
+             * of overlap are pretty slim. */
+            for (i = literals->count - 1; i >= 0; i--) {
+                if (EQ(term, entries[i].value)) {
+                    /* Dynamic literal indexes are negative, starting at -1 */
+                    return ~i;
+                }
             }
         }
 
@@ -1173,6 +1196,13 @@ Sint beamfile_add_literal(BeamFile *beam, Eterm term) {
 
             literals->entries = entries;
         }
+    } else {
+        literals->allocated = 32;
+
+        entries = erts_alloc(ERTS_ALC_T_PREPARED_CODE,
+                             literals->allocated * sizeof(*entries));
+
+        literals->entries = entries;
     }
 
     term_size = size_object(term);
@@ -1220,16 +1250,24 @@ static void move_literal_entries(BeamFile_LiteralEntry *entries, int count,
     int i;
 
     for (i = 0; i < count; i++) {
-        if (is_not_immed(entries[i].value)) {
-            ASSERT(entries[i].heap_fragments != NULL);
+        if (entries[i].heap_fragments != NULL) {
+            Eterm value = entries[i].value;
+
+#ifdef DEBUG
+            erts_literal_area_t purge_area;
+            INITIALIZE_LITERAL_PURGE_AREA(purge_area);
+#endif
+
+            ASSERT(size_object_litopt(value, &purge_area) > 0);
 
             erts_move_multi_frags(hpp, oh,
-                                  entries[i].heap_fragments, &entries[i].value,
+                                  entries[i].heap_fragments, &value,
                                   1, 1);
-            ASSERT(erts_is_literal(entries[i].value, ptr_val(entries[i].value)));
+            ASSERT(erts_is_literal(value, ptr_val(value)));
 
             free_literal_fragment(entries[i].heap_fragments);
             entries[i].heap_fragments = NULL;
+            entries[i].value = value;
         }
 
         ASSERT(entries[i].heap_fragments == NULL);
@@ -1271,7 +1309,7 @@ int iff_read_chunk(IFF_File *iff, Uint id, IFF_Chunk *chunk)
     return read_beam_chunks(iff, 1, &id, chunk);
 }
 
-void beamfile_init() {
+void beamfile_init(void) {
     Eterm suffix;
     Eterm *hp;
 
@@ -1419,7 +1457,8 @@ static int marshal_integer(BeamCodeReader *code_reader, TaggedNumber *value) {
             value->tag = TAG_q;
             value->size = 0;
 
-            value->word_value = beamfile_add_literal(code_reader->file, term);
+            value->word_value = beamfile_add_literal(code_reader->file,
+                                                     term, 1);
         } else {
             /* Result doesn't fit into a bignum. */
             value->tag = TAG_o;
@@ -1458,7 +1497,8 @@ static int marshal_allocation_list(BeamReader *reader, Sint *res) {
 
         LoadAssert(beamreader_read_tagged(reader, &val));
         LoadAssert(val.tag == TAG_u);
-        LoadAssert(val.word_value <= ERTS_SINT32_MAX / FLOAT_SIZE_OBJECT);
+        LoadAssert(val.word_value <= ERTS_SINT32_MAX /
+                   MAX(FLOAT_SIZE_OBJECT, ERL_FUN_SIZE + 1));
         number = val.word_value;
 
         switch(kind) {
@@ -1471,8 +1511,11 @@ static int marshal_allocation_list(BeamReader *reader, Sint *res) {
             sum += FLOAT_SIZE_OBJECT * number;
             break;
         case 2:
-            LoadAssert(sum <= (ERTS_SINT32_MAX - ERL_FUN_SIZE * number));
-            sum += ERL_FUN_SIZE * number;
+            LoadAssert(sum <= (ERTS_SINT32_MAX - (ERL_FUN_SIZE + 1) * number));
+
+            /* This is always a local fun, so we need to add one word to
+             * reserve space for its `FunRef`. */
+            sum += (ERL_FUN_SIZE + 1) * number;
             break;
         default:
             LoadError("Invalid allocation tag");
