@@ -1,7 +1,7 @@
 /*
  * %CopyrightBegin%
  *
- * Copyright Ericsson AB 2000-2022. All Rights Reserved.
+ * Copyright Ericsson AB 2000-2024. All Rights Reserved.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -35,24 +35,36 @@
 ** in reality are equal.
 */
 
-#ifdef ARCH_32
- /* *DO NOT USE* only for alignment. */
-#define ERTS_BINARY_STRUCT_ALIGNMENT Uint32 align__;
-#else
+enum binary_flags {
+    BIN_FLAG_MAGIC =            (1 << 0),
+    BIN_FLAG_DRV =              (1 << 1),
+
+    /* Whether the binary is writable and/or actively written to. These are
+     * valid iff the reference count is 1: any kind of observation, including
+     * copying to another process, must clear these flags. */
+    BIN_FLAG_WRITABLE =         (1 << 2),
+    BIN_FLAG_ACTIVE_WRITER =    (1 << 3),
+};
+
 #define ERTS_BINARY_STRUCT_ALIGNMENT
-#endif
 
 /* Add fields in binary_internals, otherwise the drivers crash */
 struct binary_internals {
     UWord flags;
+    /* Valid iff one of BIN_FLAG_WRITABLE and BIN_FLAG_MAGIC is set. */
+    UWord apparent_size;
     erts_refc_t refc;
+
     ERTS_BINARY_STRUCT_ALIGNMENT
 };
 
-
 typedef struct binary {
     struct binary_internals intern;
     SWord orig_size;
+
+    /* Note that this field has to be 8-byte aligned even on 32-bit
+     * platforms. Any required padding should be added at the tail of
+     * the binary_internals struct above. */
     char orig_bytes[1]; /* to be continued */
 } Binary;
 
@@ -63,6 +75,13 @@ typedef struct binary {
 #error "Update ErtsMagicBinary"
 #endif
 
+#ifdef ARCH_32
+ /* *DO NOT USE* only for alignment. */
+#define ERTS_MAGIC_BINARY_STRUCT_ALIGNMENT Uint32 align__;
+#else
+#define ERTS_MAGIC_BINARY_STRUCT_ALIGNMENT
+#endif
+
 typedef struct magic_binary ErtsMagicBinary;
 struct magic_binary {
     struct binary_internals intern;
@@ -72,7 +91,7 @@ struct magic_binary {
     ErtsAlcType_t alloc_type;
     union {
         struct {
-            ERTS_BINARY_STRUCT_ALIGNMENT
+            ERTS_MAGIC_BINARY_STRUCT_ALIGNMENT
             char data[1];
         } aligned;
         struct {
@@ -144,10 +163,6 @@ typedef union {
 				(((char *) (D)) \
 				 - offsetof(ErtsBinary, driver.binary)))
 
-/* A "magic" binary flag */
-#define BIN_FLAG_MAGIC      1
-#define BIN_FLAG_DRV        2
-
 #endif /* ERL_BINARY_H__TYPES__ */
 
 #if !defined(ERL_BINARY_H__) && !defined(ERTS_BINARY_TYPES_ONLY__)
@@ -158,118 +173,8 @@ typedef union {
 #include "erl_bif_unique.h"
 #include "erl_bits.h"
 
-/*
- * Maximum number of bytes to place in a heap binary.
- */
-
-#define ERL_ONHEAP_BIN_LIMIT 64
-
-#define ERL_SUB_BIN_SIZE (sizeof(ErlSubBin)/sizeof(Eterm))
-#define HEADER_SUB_BIN	_make_header(ERL_SUB_BIN_SIZE-2,_TAG_HEADER_SUB_BIN)
-
-/*
- * This structure represents a HEAP_BINARY.
- */
-
-typedef struct erl_heap_bin {
-    Eterm thing_word;		/* Subtag HEAP_BINARY_SUBTAG. */
-    Uint size;			/* Binary size in bytes. */
-    Eterm data[1];		/* The data in the binary. */
-} ErlHeapBin;
-
-#define heap_bin_size(num_bytes)		\
-  (sizeof(ErlHeapBin)/sizeof(Eterm) - 1 +	\
-   ((num_bytes)+sizeof(Eterm)-1)/sizeof(Eterm))
-
-#define header_heap_bin(num_bytes) \
-  _make_header(heap_bin_size(num_bytes)-1,_TAG_HEADER_HEAP_BIN)
-
-/*
- * Get the size in bytes of any type of binary.
- */
-
-#define binary_size(Bin) (binary_val(Bin)[1])
-
-#define binary_bitsize(Bin)			\
-  ((*binary_val(Bin) == HEADER_SUB_BIN) ?	\
-   ((ErlSubBin *) binary_val(Bin))->bitsize:	\
-   0)
-
-#define binary_bitoffset(Bin)			\
-  ((*binary_val(Bin) == HEADER_SUB_BIN) ?	\
-   ((ErlSubBin *) binary_val(Bin))->bitoffs:	\
-   0)
-
-/*
- * Get the pointer to the actual data bytes in a binary.
- * Works for any type of binary. Always use binary_bytes() if
- * you know that the binary cannot be a sub binary.
- *
- * Bin: input variable (Eterm)
- * Bytep: output variable (byte *)
- * Bitoffs: output variable (Uint)
- * Bitsize: output variable (Uint)
- */
-
-#define ERTS_GET_BINARY_BYTES(Bin,Bytep,Bitoffs,Bitsize)                \
-do {									\
-    Eterm* _real_bin = binary_val(Bin);		                	\
-    Uint _offs = 0;							\
-    Bitoffs = Bitsize = 0;						\
-    if (*_real_bin == HEADER_SUB_BIN) {					\
-	ErlSubBin* _sb = (ErlSubBin *) _real_bin;			\
-	_offs = _sb->offs;						\
-        Bitoffs = _sb->bitoffs;						\
-        Bitsize = _sb->bitsize;						\
-	_real_bin = binary_val(_sb->orig);	        		\
-    }									\
-    if (*_real_bin == HEADER_PROC_BIN) {				\
-	Bytep = ((ProcBin *) _real_bin)->bytes + _offs;			\
-    } else {								\
-	Bytep = (byte *)(&(((ErlHeapBin *) _real_bin)->data)) + _offs;	\
-    }									\
-} while (0)
-
-/*
- * Get the real binary from any binary type, where "real" means
- * a REFC or HEAP binary. Also get the byte and bit offset into the
- * real binary. Useful if you want to build a SUB binary from
- * any binary.
- *
- * Bin: Input variable (Eterm)
- * RealBin: Output variable (Eterm)
- * ByteOffset: Output variable (Uint)
- * BitOffset: Offset in bits (Uint)
- * BitSize: Extra bit size (Uint)
- */
-
-#define ERTS_GET_REAL_BIN(Bin, RealBin, ByteOffset, BitOffset, BitSize) \
-  do {									\
-    ErlSubBin* _sb = (ErlSubBin *) binary_val(Bin);	                \
-    if (_sb->thing_word == HEADER_SUB_BIN) {				\
-      RealBin = _sb->orig;						\
-      ByteOffset = _sb->offs;						\
-      BitOffset = _sb->bitoffs;						\
-      BitSize = _sb->bitsize;						\
-    } else {								\
-      RealBin = Bin;							\
-      ByteOffset = BitOffset = BitSize = 0;				\
-    }									\
-  } while (0)
-
-/*
- * Get a pointer to the binary bytes, for a heap or refc binary
- * (NOT sub binary).
- */
-#define binary_bytes(Bin)						\
-  (*binary_val(Bin) == HEADER_PROC_BIN ?				\
-   ((ProcBin *) binary_val(Bin))->bytes :				\
-   (ASSERT(thing_subtag(*binary_val(Bin)) == HEAP_BINARY_SUBTAG),	\
-   (byte *)(&(((ErlHeapBin *) binary_val(Bin))->data))))
-
 void erts_init_binary(void);
 
-byte* erts_get_aligned_binary_bytes_extra(Eterm, byte**, ErtsAlcType_t, unsigned extra);
 /* Used by unicode module */
 Eterm erts_bin_bytes_to_list(Eterm previous, Eterm* hp, const byte* bytes, Uint size, Uint bitoffs);
 
@@ -299,9 +204,6 @@ typedef struct {
 #define ERTS_CHK_BIN_ALIGNMENT(B) \
   do { ASSERT(!(B) || (((UWord) &((Binary *)(B))->orig_bytes[0]) & ERTS_BIN_ALIGNMENT_MASK) == ((UWord) 0)); } while(0)
 
-ERTS_GLB_INLINE byte* erts_get_aligned_binary_bytes(Eterm bin, byte** base_ptr);
-ERTS_GLB_INLINE void erts_free_aligned_binary_bytes(byte* buf);
-ERTS_GLB_INLINE void erts_free_aligned_binary_bytes_extra(byte* buf, ErtsAlcType_t);
 ERTS_GLB_INLINE Binary *erts_bin_drv_alloc_fnf(Uint size);
 ERTS_GLB_INLINE Binary *erts_bin_drv_alloc(Uint size);
 ERTS_GLB_INLINE Binary *erts_bin_nrml_alloc_fnf(Uint size);
@@ -335,26 +237,6 @@ ERTS_GLB_INLINE erts_atomic_t *erts_binary_to_magic_indirection(Binary *bp);
 
 #include <stddef.h> /* offsetof */
 
-ERTS_GLB_INLINE byte*
-erts_get_aligned_binary_bytes(Eterm bin, byte** base_ptr)
-{
-    return erts_get_aligned_binary_bytes_extra(bin, base_ptr, ERTS_ALC_T_TMP, 0);
-}
-
-ERTS_GLB_INLINE void
-erts_free_aligned_binary_bytes_extra(byte* buf, ErtsAlcType_t allocator)
-{
-    if (buf) {
-	erts_free(allocator, (void *) buf);
-    }
-}
-
-ERTS_GLB_INLINE void
-erts_free_aligned_binary_bytes(byte* buf)
-{
-    erts_free_aligned_binary_bytes_extra(buf,ERTS_ALC_T_TMP);
-}
-
 /* Explicit extra bytes allocated to counter buggy drivers.
 ** These extra bytes where earlier (< R13B04) added by an alignment-bug
 ** in this code. Do we dare remove this in some major release (R14?) maybe?
@@ -523,6 +405,7 @@ erts_create_magic_binary_x(Uint size, int (*destructor)(Binary *),
 	erts_alloc_n_enomem(ERTS_ALC_T2N(alloc_type), bsize);
     ERTS_CHK_BIN_ALIGNMENT(bptr);
     bptr->intern.flags = BIN_FLAG_MAGIC;
+    bptr->intern.apparent_size = size;
     bptr->orig_size = unaligned ? ERTS_MAGIC_BIN_UNALIGNED_ORIG_SIZE(size)
                                 : ERTS_MAGIC_BIN_ORIG_SIZE(size);
     erts_refc_init(&bptr->intern.refc, 0);
