@@ -1,7 +1,7 @@
 /*
  * %CopyrightBegin%
  *
- * Copyright Ericsson AB 2000-2022. All Rights Reserved.
+ * Copyright Ericsson AB 2000-2023. All Rights Reserved.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -42,16 +42,35 @@ typedef struct erl_fun_entry {
     int old_index;                  /* Old style index */
 
     erts_refc_t refc;               /* Reference count: One for code + one for
-                                     * each fun object in each process. */
+                                     * each FunRef. */
     ErtsCodePtr pend_purge_address; /* Address during a pending purge */
 } ErlFunEntry;
 
+/* Reference-holding structure for funs. As these normally live in the literal
+ * area of their module instance and are shared with all lambdas pointing to
+ * the same function, having it separated like this saves us from having to
+ * reference-count every single lambda.
+ *
+ * These references copied onto the heap under some circumstances, for example
+ * trips through ETS or the external term format, but generally they'll live
+ * off-heap. */
+typedef struct erl_fun_ref {
+    Eterm thing_word;
+    ErlFunEntry *entry;
+    struct erl_off_heap_header *next;
+} FunRef;
+
+#define ERL_FUN_REF_SIZE ((sizeof(FunRef)/sizeof(Eterm)))
+#define HEADER_FUN_REF _make_header(ERL_FUN_REF_SIZE-1,_TAG_HEADER_FUN_REF)
+
 /* This structure represents a 'fun' (lambda), whether local or external. It is
  * stored on process heaps, and has variable size depending on the size of the
  * environment. */
 
 typedef struct erl_fun_thing {
-    Eterm thing_word;       /* Subtag FUN_SUBTAG. */
+    /* The header contains FUN_SUBTAG, arity, and the size of the environment,
+     * the latter being zero for external funs and non-zero for local ones. */
+    Eterm thing_word;
 
     union {
         /* Both `ErlFunEntry` and `Export` begin with an `ErtsDispatchable`, so
@@ -59,35 +78,34 @@ typedef struct erl_fun_thing {
          * pointer to improve performance. */
         ErtsDispatchable *disp;
 
-        /* Pointer to function entry, valid iff `creator != am_external`.*/
+        /* Pointer to function entry, valid iff this is local fun. */
         ErlFunEntry *fun;
 
-        /* Pointer to export entry, valid iff `creator == am_external`.*/
+        /* Pointer to export entry, valid iff this is an external fun. */
         Export *exp;
     } entry;
 
-    /* Next off-heap object, must be NULL when this is an external fun. */
-    struct erl_off_heap_header *next;
-
-    byte arity;             /* The _apparent_ arity of the fun. */
-    byte num_free;          /* Number of free variables (in env). */
-
-    /* -- The following may be compound Erlang terms ---------------------- */
-    Eterm creator;          /* Pid of creator process (contains node). */
-    Eterm env[1];           /* Environment (free variables). */
+    /* Environment (free variables), may be compound terms.
+     *
+     * External funs lack this altogether, and local funs _always_ reference a
+     * `FunRef` just past the last free variable. This ensures that the
+     * `ErlFunEntry` above will always be valid. */
+    Eterm env[];
 } ErlFunThing;
 
-#define is_local_fun(FunThing) ((FunThing)->creator != am_external)
-#define is_external_fun(FunThing) ((FunThing)->creator == am_external)
+#define is_external_fun(FunThing) (fun_env_size(FunThing) == 0)
+#define is_local_fun(FunThing) (!is_external_fun(FunThing))
 
-/* ERL_FUN_SIZE does _not_ include space for the environment */
-#define ERL_FUN_SIZE ((sizeof(ErlFunThing)/sizeof(Eterm))-1)
+#define fun_arity(FunThing)                                                   \
+    (((FunThing)->thing_word >> FUN_HEADER_ARITY_OFFS) & 0xFF)
+#define fun_env_size(FunThing)                                                \
+    ((FunThing)->thing_word >> FUN_HEADER_ENV_SIZE_OFFS)
+#define fun_num_free(FunThing)                                                \
+    (ASSERT(is_local_fun(FunThing)), fun_env_size(FunThing) - 1)
 
-ErlFunThing *erts_new_export_fun_thing(Eterm **hpp, Export *exp, int arity);
-ErlFunThing *erts_new_local_fun_thing(Process *p,
-                                      ErlFunEntry *fe,
-                                      int arity,
-                                      int num_free);
+/* ERL_FUN_SIZE does _not_ include space for the environment which is a
+ * C99-style flexible array */
+#define ERL_FUN_SIZE ((sizeof(ErlFunThing)/sizeof(Eterm)))
 
 void erts_init_fun_table(void);
 void erts_fun_info(fmtfn_t, void *);
@@ -97,14 +115,14 @@ int erts_fun_table_sz(void);
 ErlFunEntry* erts_put_fun_entry2(Eterm mod, int old_uniq, int old_index,
                                  const byte* uniq, int index, int arity);
 
-const ErtsCodeMFA *erts_get_fun_mfa(const ErlFunEntry *fe);
+const ErtsCodeMFA *erts_get_fun_mfa(const ErlFunEntry *fe, ErtsCodeIndex ix);
 
-void erts_set_fun_code(ErlFunEntry *fe, ErtsCodePtr address);
+void erts_set_fun_code(ErlFunEntry *fe, ErtsCodeIndex ix, ErtsCodePtr address);
 
 ERTS_GLB_INLINE
 ErtsCodePtr erts_get_fun_code(ErlFunEntry *fe, ErtsCodeIndex ix);
 
-int erts_is_fun_loaded(const ErlFunEntry* fe);
+int erts_is_fun_loaded(const ErlFunEntry* fe, ErtsCodeIndex ix);
 
 void erts_erase_fun_entry(ErlFunEntry* fe);
 void erts_cleanup_funs(ErlFunThing* funp);
@@ -116,6 +134,10 @@ void erts_fun_purge_abort_finalize(ErlFunEntry **funs, Uint no);
 void erts_fun_purge_complete(ErlFunEntry **funs, Uint no);
 void erts_dump_fun_entries(fmtfn_t, void *);
 
+
+void erts_fun_start_staging(void);
+void erts_fun_end_staging(int commit);
+
 #if ERTS_GLB_INLINE_INCL_FUNC_DEF
 
 ERTS_GLB_INLINE
