@@ -1,7 +1,7 @@
 /*
  * %CopyrightBegin%
  *
- * Copyright Ericsson AB 1997-2023. All Rights Reserved.
+ * Copyright Ericsson AB 1997-2025. All Rights Reserved.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -53,6 +53,7 @@
 #include "erl_proc_sig_queue.h"
 #include "beam_load.h"
 #include "erl_global_literals.h"
+#include "erl_iolist.h"
 
 #include "jit/beam_asm.h"
 
@@ -66,67 +67,8 @@
 #define ERTS_DEFAULT_DCPU_SCHED_STACK_SIZE 40
 #define ERTS_DEFAULT_DIO_SCHED_STACK_SIZE 40
 
-/*
- * The variables below (prefixed with etp_) are for erts/etc/unix/etp-commands
- * only. Do not remove even though they aren't used elsewhere in the emulator!
- */
-const int etp_smp_compiled = 1;
-const int etp_thread_compiled = 1;
-const char etp_erts_version[] = ERLANG_VERSION;
-const char etp_otp_release[] = ERLANG_OTP_RELEASE;
-const char etp_arch[] = ERLANG_ARCHITECTURE;
-#if ERTS_ENABLE_KERNEL_POLL
-const int erts_use_kernel_poll = 1;
-const int etp_kernel_poll_support = 1;
-#else
-const int erts_use_kernel_poll = 0;
-const int etp_kernel_poll_support = 0;
-#endif
-#if defined(ARCH_64)
-const int etp_arch_bits = 64;
-#elif defined(ARCH_32)
-const int etp_arch_bits = 32;
-#else
-# error "Not 64-bit, nor 32-bit arch"
-#endif
-#ifdef BEAMASM
-const int etp_beamasm = 1;
-#else
-const int etp_beamasm = 0;
-#endif
-#ifdef DEBUG
-const int etp_debug_compiled = 1;
-#else
-const int etp_debug_compiled = 0;
-#endif
-#ifdef ERTS_ENABLE_LOCK_COUNT
-const int etp_lock_count = 1;
-#else
-const int etp_lock_count = 0;
-#endif
-#ifdef ERTS_ENABLE_LOCK_CHECK
-const int etp_lock_check = 1;
-#else
-const int etp_lock_check = 0;
-#endif
-const int etp_endianness = ERTS_ENDIANNESS;
-const Eterm etp_ref_header = ERTS_REF_THING_HEADER;
-#ifdef ERTS_MAGIC_REF_THING_HEADER
-const Eterm etp_magic_ref_header = ERTS_MAGIC_REF_THING_HEADER;
-#else
-const Eterm etp_magic_ref_header = ERTS_REF_THING_HEADER;
-#endif
-const Eterm etp_the_non_value = THE_NON_VALUE;
-#ifdef TAG_LITERAL_PTR
-const Eterm etp_ptr_mask = (~(Eterm)7);
-#else
-const Eterm etp_ptr_mask = (~(Eterm)3);
-#endif
-#ifdef ERTS_HOLE_MARKER
-const Eterm etp_hole_marker = ERTS_HOLE_MARKER;
-#else
-const Eterm etp_hole_marker = 0;
-#endif
+#define ERTS_HALT_FLUSH_TIMEOUT_EXIT_CODE 255
+ErtsMonotonicTime erts_halt_flush_timeout = -1; /* negative value -> infinity */
 
 static int modified_sched_thread_suggested_stack_size = 0;
 
@@ -199,6 +141,7 @@ int erts_no_crash_dump = 0;	/* Use -d to suppress crash dump. */
 int erts_no_line_info = 0;	/* -L: Don't load line information */
 
 #ifdef BEAMASM
+Uint erts_coverage_mode = ERTS_COV_NONE; /* -JPcover: Enable coverage */
 int erts_jit_asm_dump = 0;	/* -JDdump: Dump assembly code */
 #endif
 
@@ -275,7 +218,7 @@ static ERTS_INLINE void
 set_default_time_adj(int *time_correction_p, ErtsTimeWarpMode *time_warp_mode_p)
 {
     *time_correction_p = 1;
-    *time_warp_mode_p = ERTS_NO_TIME_WARP_MODE;
+    *time_warp_mode_p = ERTS_MULTI_TIME_WARP_MODE;
     if (!erts_check_time_adj_support(*time_correction_p,
 				     *time_warp_mode_p)) {
 	*time_correction_p = 0;
@@ -343,7 +286,6 @@ erl_init(int ncpu,
     BIN_VH_MIN_SIZE = erts_next_heap_size(BIN_VH_MIN_SIZE, 0);
 
     erts_init_trace();
-    erts_init_bits();
     erts_code_ix_init();
     erts_init_fun_table();
     init_atom_table();
@@ -359,6 +301,7 @@ erl_init(int ncpu,
     init_emulator();
     erts_ptab_init(); /* Must be after init_emulator() */
     erts_init_binary(); /* Must be after init_emulator() */
+    erts_init_iolist(); /* Must be after init_emulator() */
     erts_bp_init();
     init_db(db_spin_count); /* Must be after init_emulator */
     erts_init_node_tables(node_tab_delete_delay);
@@ -388,6 +331,7 @@ erl_init(int ncpu,
     erl_nif_init();
     erts_msacc_init();
     beamfile_init();
+    erts_late_init_external();
 }
 
 static Eterm
@@ -433,9 +377,14 @@ erl_first_process_otp(char* mod_name, int argc, char** argv)
     hp = HAlloc(&parent, argc*2 + 4);
     args = NIL;
     for (i = argc-1; i >= 0; i--) {
-	int len = sys_strlen(argv[i]);
-	args = CONS(hp, new_binary(&parent, (byte*)argv[i], len), args);
-	hp += 2;
+        size_t len = sys_strlen(argv[i]);
+
+        args = CONS(hp,
+                    erts_new_binary_from_data(&parent,
+                                              len,
+                                              (byte*)argv[i]),
+                    args);
+        hp += 2;
     }
     boot_mod = erts_atom_put((byte *) mod_name, sys_strlen(mod_name),
                              ERTS_ATOM_ENC_LATIN1, 1);
@@ -631,6 +580,7 @@ void erts_usage(void)
 	       H_DEFAULT_MAX_SIZE);
     erts_fprintf(stderr, "-hmaxk bool    enable or disable kill at max heap size (default true)\n");
     erts_fprintf(stderr, "-hmaxel bool   enable or disable error_logger report at max heap size (default true)\n");
+    erts_fprintf(stderr, "-hmaxib bool   enable or disable including off-heap binaries into max heap size (default false)\n");
     erts_fprintf(stderr, "-hpds size     set initial process dictionary size (default %d)\n",
 	       erts_pd_initial_size);
     erts_fprintf(stderr, "-hmqd  val     set default message queue data flag for processes;\n");
@@ -651,7 +601,9 @@ void erts_usage(void)
 
 #ifdef BEAMASM
     erts_fprintf(stderr, "-JDdump bool   enable or disable dumping of generated assembly code for each module loaded\n");
+    erts_fprintf(stderr, "-JPcover true|false|line|line_counters|function|function_counters  enable or disable instrumentation for coverage\n");
     erts_fprintf(stderr, "-JPperf true|false|dump|map|fp|no_fp   enable or disable support for perf on Linux\n");
+    erts_fprintf(stderr, "-JMsingle bool enable the use of single-mapped RWX memory for JIT:ed code\n");
     erts_fprintf(stderr, "\n");
 #endif
 
@@ -765,6 +717,8 @@ void erts_usage(void)
     erts_fprintf(stderr, "-zosrl number  set outstanding requests limit for system processes,\n");
     erts_fprintf(stderr, "               valid range [1-%d]\n",
                  ERTS_MAX_PROCESSES);
+    erts_fprintf(stderr, "-zhft timeout  set halt flush timeout,\n");
+    erts_fprintf(stderr, "               valid range [0-%d] or infinity\n", INT_MAX);
     erts_fprintf(stderr, "\n");
     erts_fprintf(stderr, "Note that if the emulator is started with erlexec (typically\n");
     erts_fprintf(stderr, "from the erl script), these flags should be specified with +.\n");
@@ -1579,6 +1533,8 @@ erl_start(int argc, char **argv)
              * h|max   - max_heap_size
              * h|maxk  - max_heap_kill
              * h|maxel - max_heap_error_logger
+             * h|maxib - map_heap_include_shared_binaries
+             *
 	     *
 	     */
 	    if (has_prefix("mbs", sub_param)) {
@@ -1641,6 +1597,17 @@ erl_start(int argc, char **argv)
 		    erts_usage();
 		}
 		VERBOSE(DEBUG_SYSTEM, ("using max heap log %d\n", H_MAX_FLAGS));
+            } else if (has_prefix("maxib", sub_param)) {
+                arg = get_arg(sub_param+5, argv[i+1], &i);
+                if (sys_strcmp(arg,"true") == 0) {
+                    H_MAX_FLAGS |= MAX_HEAP_SIZE_INCLUDE_OH_BINS;
+                } else if (sys_strcmp(arg,"false") == 0) {
+                    H_MAX_FLAGS &= ~MAX_HEAP_SIZE_INCLUDE_OH_BINS;
+                } else {
+                    erts_fprintf(stderr, "bad max heap include bins %s\n", arg);
+                    erts_usage();
+                }
+                VERBOSE(DEBUG_SYSTEM, ("using max heap log %d\n", H_MAX_FLAGS));
 	    } else if (has_prefix("max", sub_param)) {
                 Sint hMaxSize;
                 char *rest;
@@ -1751,6 +1718,41 @@ erl_start(int argc, char **argv)
                 erts_fprintf(stderr, "+JPperf is not supported on this platform\n");
                 erts_usage();
 #endif
+                } else if (has_prefix("cover", sub_param)) {
+                    arg = get_arg(sub_param+5, argv[i + 1], &i);
+                    if (sys_strcmp(arg, "true") == 0) {
+                        erts_coverage_mode = ERTS_COV_LINE_COUNTERS;
+                    } else if (sys_strcmp(arg, "false") == 0) {
+                        erts_coverage_mode = ERTS_COV_NONE;
+                    } else if (sys_strcmp(arg, "function") == 0) {
+                        erts_coverage_mode = ERTS_COV_FUNCTION;
+                    } else if (sys_strcmp(arg, "function_counters") == 0) {
+                        erts_coverage_mode = ERTS_COV_FUNCTION_COUNTERS;
+                    } else if (sys_strcmp(arg, "line") == 0) {
+                        erts_coverage_mode = ERTS_COV_LINE;
+                    } else if (sys_strcmp(arg, "line_counters") == 0) {
+                        erts_coverage_mode = ERTS_COV_LINE_COUNTERS;
+                    } else {
+                        erts_fprintf(stderr, "bad +JPcover flag %s\n", arg);
+                        erts_usage();
+                    }
+                }
+                break;
+            case 'M':
+                sub_param++;
+                if (has_prefix("single", sub_param)) {
+                    arg = get_arg(sub_param+6, argv[i + 1], &i);
+                    if (sys_strcmp(arg, "true") == 0) {
+                        erts_jit_single_map = 1;
+                    } else if (sys_strcmp(arg, "false") == 0) {
+                        erts_jit_single_map = 0;
+                    } else {
+                        erts_fprintf(stderr, "bad +JMsingle support flag %s\n", arg);
+                        erts_usage();
+                    }
+                } else {
+                    erts_fprintf(stderr, "bad +JM sub-option %s\n", arg);
+                    erts_usage();
                 }
                 break;
             default:
@@ -2220,7 +2222,7 @@ erl_start(int argc, char **argv)
 	    /* suggested stack size (Kilo Words) for threads in thread pool */
 	    arg = get_arg(argv[i]+2, argv[i+1], &i);
 	    erts_async_thread_suggested_stack_size = atoi(arg);
-	    
+
 	    if ((erts_async_thread_suggested_stack_size
 		 < ERTS_ASYNC_THREAD_MIN_STACK_SIZE)
 		|| (erts_async_thread_suggested_stack_size >
@@ -2375,6 +2377,22 @@ erl_start(int argc, char **argv)
                 }
                 sys_proc_outst_req_lim = (Uint) val;
             }
+            else if (has_prefix("hft", sub_param)) {
+                long val;
+		arg = get_arg(sub_param+3, argv[i+1], &i);
+                if (strcmp(arg, "infinity") == 0) {
+                    erts_halt_flush_timeout = -1;
+                }
+                else {
+                    errno = 0;
+                    val = strtol(arg, NULL, 10);
+                    if (errno != 0 || val < 0 || INT_MAX < val) {
+                        erts_fprintf(stderr, "Invalid halt flush timeout %s\n", arg);
+                        erts_usage();
+                    }
+                    erts_halt_flush_timeout = (ErtsMonotonicTime) val;
+                }
+            }
 	    else {
 		erts_fprintf(stderr, "bad -z option %s\n", argv[i]);
 		erts_usage();
@@ -2471,8 +2489,7 @@ erl_start(int argc, char **argv)
 
     {
 	/*
-	 * The erts_code_purger and the erts_literal_area_collector
-	 * system processes are *always* alive. If they terminate
+	 * System processes that are *always* alive. If they terminate
 	 * they bring the whole VM down.
 	 */
 	Eterm pid;
@@ -2484,7 +2501,7 @@ erl_start(int argc, char **argv)
 	    = (Process *) erts_ptab_pix2intptr_ddrb(&erts_proc,
 						    internal_pid_index(pid));
 	ASSERT(erts_code_purger && erts_code_purger->common.id == pid);
-	erts_proc_inc_refc(erts_code_purger); 
+	erts_proc_inc_refc(erts_code_purger);
 
 	pid = erl_system_process_otp(erts_init_process_id,
                                      "erts_literal_area_collector",
@@ -2525,6 +2542,16 @@ erl_start(int argc, char **argv)
 	ASSERT(erts_dirty_process_signal_handler_max
 	       && erts_dirty_process_signal_handler_max->common.id == pid);
 	erts_proc_inc_refc(erts_dirty_process_signal_handler_max);
+
+        pid = erl_system_process_otp(erts_init_process_id,
+                                     "erts_trace_cleaner", !0,
+                                     PRIORITY_NORMAL);
+        erts_trace_cleaner
+            = (Process *) erts_ptab_pix2intptr_ddrb(&erts_proc,
+                                                    internal_pid_index(pid));
+        ASSERT(erts_trace_cleaner && erts_trace_cleaner->common.id == pid);
+        erts_proc_inc_refc(erts_trace_cleaner);
+
     }
 
     erts_start_schedulers();
@@ -2561,7 +2588,7 @@ __decl_noreturn void erts_thr_fatal_error(int err, const char *what)
 
 
 static void
-system_cleanup(int flush_async)
+system_cleanup(int flush)
 {
     /*
      * Make sure only one thread exits the runtime system.
@@ -2591,24 +2618,43 @@ system_cleanup(int flush_async)
      *    (in threaded non smp case).
      */
 
-    if (!flush_async
-	|| !erts_initialized
-	)
+    if (!flush || !erts_initialized)
 	return;
 
+    /*
+     * We only flush as a result of calling erts_halt() (which in turn
+     * is called from the erlang:halt() BIF when flushing is enabled);
+     * otherwise, flushing wont work properly. If erts_halt() has
+     * been called, 'erts_halt_code' won't equal INT_MIN...
+     */
+    ASSERT(erts_halt_code != INT_MIN);
+
+    /*
+     * Nif on-halt handlers may have been added after we initiated
+     * a halt. If so, make sure that these late added handlers are
+     * executed as well..
+     */
+    erts_nif_execute_on_halt();
+
 #ifdef ERTS_ENABLE_LOCK_CHECK
     erts_lc_check_exact(NULL, 0);
 #endif
 
     erts_exit_flush_async();
+
+    /*
+     * Wait for all NIF calls with delayed halt functionality
+     * enabled to complete before we continue...
+     */
+    erts_nif_wait_calls();
 }
 
 static int erts_exit_code;
 
 static __decl_noreturn void __noreturn
-erts_exit_vv(int n, int flush_async, const char *fmt, va_list args1, va_list args2)
+erts_exit_vv(int n, int flush, const char *fmt, va_list args1, va_list args2)
 {
-    system_cleanup(flush_async);
+    system_cleanup(flush);
 
     if (fmt != NULL && *fmt != '\0')
 	erl_error(fmt, args2);	/* Print error message. */
@@ -2621,25 +2667,29 @@ erts_exit_vv(int n, int flush_async, const char *fmt, va_list args1, va_list arg
 	erl_crash_dump_v((char*) NULL, 0, fmt, args1);
     }
 
-    erts_exit_epilogue();
+    erts_exit_epilogue(flush);
 }
 
-__decl_noreturn void __noreturn erts_exit_epilogue(void)
+__decl_noreturn void __noreturn erts_exit_epilogue(int flush)
 {
     int n = erts_exit_code;
 
     sys_tty_reset(n);
 
+#ifdef ERTS_GCOV
+    flush = !0; /* otherwise we get no results... */
+#endif
+
     if (n == ERTS_INTR_EXIT)
-	exit(0);
+	(void) (flush ? exit(0) : _exit(0));
     else if (n == ERTS_DUMP_EXIT)
 	ERTS_EXIT_AFTER_DUMP(1);
     else if (n == ERTS_ERROR_EXIT || n == ERTS_ABORT_EXIT)
         abort();
-    exit(n);
+    (void) (flush ? exit(n) : _exit(n));
 }
 
-/* Exit without flushing async threads */
+/* Exit without flushing */
 __decl_noreturn void __noreturn erts_exit(int n, const char *fmt, ...)
 {
     va_list args1, args2;
@@ -2650,8 +2700,12 @@ __decl_noreturn void __noreturn erts_exit(int n, const char *fmt, ...)
     va_end(args1);
 }
 
-/* Exit after flushing async threads */
-__decl_noreturn void __noreturn erts_flush_async_exit(int n, char *fmt, ...)
+/*
+ * Exit after flushing. This is a continuation of erts_halt() and wont
+ * work properly if called by its own without proper initialization
+ * as made in erts_halt().
+ */
+__decl_noreturn void __noreturn erts_flush_exit(int n, char *fmt, ...)
 {
     va_list args1, args2;
     va_start(args1, fmt);
@@ -2660,3 +2714,8 @@ __decl_noreturn void __noreturn erts_flush_async_exit(int n, char *fmt, ...)
     va_end(args2);
     va_end(args1);
 }
+
+void erts_halt_flush_timeout_callback(void *arg)
+{
+    _exit(ERTS_HALT_FLUSH_TIMEOUT_EXIT_CODE);
+}
