@@ -95,17 +95,76 @@
 #  define ERTS_PROC_SIG_HARD_DEBUG_SIGQ_BUFFERS
 #endif
 
+#define ERTS_HDBG_PRIVQ_LEN__(P)                                        \
+    do {                                                                \
+        Sint len = 0;                                                   \
+        ErtsMessage *sig;                                               \
+        ERTS_LC_ASSERT(erts_proc_lc_my_proc_locks((P))                  \
+                       & ERTS_PROC_LOCK_MAIN);                          \
+        for (sig = (P)->sig_qs.first; sig; sig = sig->next) {           \
+            if (ERTS_SIG_IS_MSG(sig))                                   \
+                len++;                                                  \
+        }                                                               \
+        ERTS_ASSERT((P)->sig_qs.mq_len == len);                         \
+        for (sig = (P)->sig_qs.cont, len = 0; sig; sig = sig->next) {   \
+            if (ERTS_SIG_IS_MSG(sig)) {                                 \
+                len++;                                                  \
+            }                                                           \
+            else {                                                      \
+                ErtsNonMsgSignal *nmsig = (ErtsNonMsgSignal *) sig;     \
+                ERTS_ASSERT(nmsig->mlenoffs == len);                    \
+                len = 0;                                                \
+            }                                                           \
+        }                                                               \
+        ERTS_ASSERT((P)->sig_qs.mlenoffs == len);                       \
+    } while (0)
+#define ERTS_HDBG_INQ_LEN__(Q)                                          \
+    do {                                                                \
+        Sint len = 0;                                                   \
+        ErtsMessage *sig;                                               \
+        for (sig = (Q)->first; sig; sig = sig->next) {                  \
+            if (ERTS_SIG_IS_MSG(sig)) {                                 \
+                len++;                                                  \
+            }                                                           \
+            else {                                                      \
+                ErtsNonMsgSignal *nmsig = (ErtsNonMsgSignal *) sig;     \
+                ERTS_ASSERT(nmsig->mlenoffs == len);                    \
+                len = 0;                                                \
+            }                                                           \
+        }                                                               \
+        ERTS_ASSERT((Q)->mlenoffs == len);                              \
+    } while (0)
+
+#ifdef ERTS_PROC_SIG_HARD_DEBUG_SIGQ_MSG_LEN
+#define ERTS_HDBG_PRIVQ_LEN(P) ERTS_HDBG_PRIVQ_LEN__((P))
+#define ERTS_HDBG_INQ_LEN(Q) ERTS_HDBG_INQ_LEN((Q))
+#else
+#define ERTS_HDBG_PRIVQ_LEN(P)
+#define ERTS_HDBG_INQ_LEN(Q)
+#endif
+
+
 struct erl_mesg;
 struct erl_dist_external;
 
+#define ERTS_SIGNAL_COMMON_FIELDS__     \
+    struct erl_mesg *next;              \
+    union {                             \
+        struct erl_mesg **next;         \
+        void *attachment;               \
+    } specific;                         \
+    Eterm tag
+
+
 typedef struct {
-    struct erl_mesg *next;
-    union {
-        struct erl_mesg **next;
-        void *attachment;
-    } specific;
-    Eterm tag;
+    ERTS_SIGNAL_COMMON_FIELDS__;
 } ErtsSignalCommon;
+
+typedef struct {
+    ERTS_SIGNAL_COMMON_FIELDS__;
+    Sint mlenoffs; /* Number of msg sigs preceeding the non-msg sig */
+} ErtsNonMsgSignal;
+
 /*
  * Note that not all signal are handled using this functionality!
  */
@@ -157,30 +216,42 @@ typedef struct {
     ERTS_SIG_Q_TYPE_MAX
 
 #define ERTS_SIG_IS_DIST_ALIAS_MSG_TAG(Tag)                          \
-    ((Tag) == ERTS_PROC_SIG_MAKE_TAG(ERTS_SIG_Q_OP_ALIAS_MSG,        \
-                                     ERTS_SIG_Q_TYPE_DIST,           \
-                                     0))
+    (((Tag) & (ERTS_PROC_SIG_TYPE_MASK                               \
+               | ERTS_PROC_SIG_OP_MASK                               \
+               | ERTS_PROC_SIG_BASE_TAG_MASK))                       \
+     == ERTS_PROC_SIG_MAKE_TAG(ERTS_SIG_Q_OP_ALIAS_MSG,              \
+                               ERTS_SIG_Q_TYPE_DIST,                 \
+                               0))
 #define ERTS_SIG_IS_DIST_ALIAS_MSG(sig)                              \
     ERTS_SIG_IS_DIST_ALIAS_MSG_TAG(((ErtsSignal *) (sig))->common.tag)
 
 #define ERTS_SIG_IS_OFF_HEAP_ALIAS_MSG_TAG(Tag)                      \
-    ((Tag) == ERTS_PROC_SIG_MAKE_TAG(ERTS_SIG_Q_OP_ALIAS_MSG,        \
-                                     ERTS_SIG_Q_TYPE_OFF_HEAP,       \
-                                     0))
+    (((Tag) & (ERTS_PROC_SIG_TYPE_MASK                               \
+               | ERTS_PROC_SIG_OP_MASK                               \
+               | ERTS_PROC_SIG_BASE_TAG_MASK))                       \
+     == ERTS_PROC_SIG_MAKE_TAG(ERTS_SIG_Q_OP_ALIAS_MSG,              \
+                               ERTS_SIG_Q_TYPE_OFF_HEAP,             \
+                               0))
 #define ERTS_SIG_IS_OFF_HEAP_ALIAS_MSG(sig)                          \
     ERTS_SIG_IS_OFF_HEAP_ALIAS_MSG_TAG(((ErtsSignal *) (sig))->common.tag)
 
 #define ERTS_SIG_IS_HEAP_ALIAS_MSG_TAG(Tag)                          \
-    ((Tag) == ERTS_PROC_SIG_MAKE_TAG(ERTS_SIG_Q_OP_ALIAS_MSG,        \
-                                     ERTS_SIG_Q_TYPE_HEAP,           \
-                                     0))
+    (((Tag) & (ERTS_PROC_SIG_TYPE_MASK                               \
+               | ERTS_PROC_SIG_OP_MASK                               \
+               | ERTS_PROC_SIG_BASE_TAG_MASK))                       \
+     == ERTS_PROC_SIG_MAKE_TAG(ERTS_SIG_Q_OP_ALIAS_MSG,              \
+                               ERTS_SIG_Q_TYPE_HEAP,                 \
+                               0))
 #define ERTS_SIG_IS_HEAP_ALIAS_MSG(sig)                              \
     ERTS_SIG_IS_HEAP_ALIAS_MSG_TAG(((ErtsSignal *) (sig))->common.tag)
 
 #define ERTS_SIG_IS_HEAP_FRAG_ALIAS_MSG_TAG(Tag)                     \
-    ((Tag) == ERTS_PROC_SIG_MAKE_TAG(ERTS_SIG_Q_OP_ALIAS_MSG,        \
-                                     ERTS_SIG_Q_TYPE_HEAP_FRAG,      \
-                                     0))
+    (((Tag) & (ERTS_PROC_SIG_TYPE_MASK                               \
+               | ERTS_PROC_SIG_OP_MASK                               \
+               | ERTS_PROC_SIG_BASE_TAG_MASK))                       \
+     == ERTS_PROC_SIG_MAKE_TAG(ERTS_SIG_Q_OP_ALIAS_MSG,              \
+                               ERTS_SIG_Q_TYPE_HEAP_FRAG,            \
+                               0))
 #define ERTS_SIG_IS_HEAP_FRAG_ALIAS_MSG(sig)                         \
     ERTS_SIG_IS_HEAP_FRAG_ALIAS_MSG_TAG(((ErtsSignal *) (sig))->common.tag)
 
@@ -193,17 +264,13 @@ typedef struct {
 #define ERTS_RECV_MARKER_PASS_MAX 4
 
 typedef struct {
-    ErtsSignalCommon common;
+    ErtsNonMsgSignal common;
     Eterm from;
     Uint64 id;
 } ErtsSigUnlinkOp;
 
 #define ERTS_SIG_HANDLE_REDS_MAX_PREFERED (CONTEXT_REDS/40)
 
-#ifdef ERTS_SUPPORT_OLD_RECV_MARK_INSTRS
-extern Eterm erts_old_recv_marker_id;
-#endif
-
 #ifdef ERTS_PROC_SIG_HARD_DEBUG
 #  define ERTS_HDBG_CHECK_SIGNAL_IN_QUEUE(P, B) \
     ERTS_HDBG_CHECK_SIGNAL_IN_QUEUE__((P), (B), "")
@@ -303,16 +370,14 @@ int erts_proc_sig_queue_force_buffers(Process*);
                      << ERTS_SIG_Q_XTRA_SHIFT),                 \
                   _TAG_HEADER_EXTERNAL_PID))
 
-
-/*
- * ERTS_SIG_Q_OP_MSGQ_LEN_OFFS_MARK is not an actual
- * operation. We keep it at the top of the OP range,
- * larger than ERTS_SIG_Q_OP_MAX.
- */
-#define ERTS_SIG_Q_OP_MSGQ_LEN_OFFS_MARK ERTS_SIG_Q_OP_MASK
-
-#define ERTS_PROC_SIG_MSGQ_LEN_OFFS_MARK \
-    ERTS_PROC_SIG_MAKE_TAG(ERTS_SIG_Q_OP_MSGQ_LEN_OFFS_MARK,0,0)
+#define ERTS_PROC_SIG_BASE_TAG_MASK \
+    ((1 << _HEADER_ARITY_OFFS)-1)
+#define ERTS_PROC_SIG_OP_MASK \
+    (ERTS_SIG_Q_OP_MASK << (ERTS_SIG_Q_OP_SHIFT + _HEADER_ARITY_OFFS))
+#define ERTS_PROC_SIG_TYPE_MASK \
+    (ERTS_SIG_Q_TYPE_MASK << (ERTS_SIG_Q_TYPE_SHIFT + _HEADER_ARITY_OFFS))
+#define ERTS_PROC_SIG_XTRA_MASK \
+    (ERTS_SIG_Q_XTRA_MASK << (ERTS_SIG_Q_XTRA_SHIFT + _HEADER_ARITY_OFFS))
 
 struct dist_entry_;
 
@@ -866,16 +931,12 @@ erts_proc_sig_send_is_alive_request(Process *c_p, Eterm to,
  * @param[in]     item_ix       Info index array to pass to
  *                              erts_process_info()
  *
- * @param[in]     len           Length of info index array
- *
- * @param[in]     need_msgq_len Non-zero if message queue
- *                              length is needed; otherwise,
- *                              zero. If non-zero, sig_qs.len
- *                              will be set to correspond
- *                              to the message queue length
- *                              before call to
+ * @param[in]     item_extra    Extra terms array to pass to
  *                              erts_process_info()
  *
+ * @param[in]     len           Length of info index array and
+ *                              extra array if such is provided
+ *
  * @param[in]     flags         Flags to pass to
  *                              erts_process_info()
  *
@@ -892,8 +953,8 @@ int
 erts_proc_sig_send_process_info_request(Process *c_p,
                                         Eterm to,
                                         int *item_ix,
+                                        Eterm *item_extra,
                                         int len,
-                                        int need_msgq_len,
                                         int flags,
                                         Uint reserve_size,
                                         Eterm ref);
@@ -937,20 +998,24 @@ erts_proc_sig_send_sync_suspend(Process *c_p, Eterm to,
  * @brief Send an 'rpc' signal to a process.
  *
  * The function 'func' will be executed in the
- * context of the receiving process. A response
- * message '{Ref, Result}' is sent to the sender
- * when 'func' has been called. 'Ref' is the reference
- * returned by this function and 'Result' is the
- * term returned by 'func'. If the return value of
- * 'func' is not an immediate term, 'func' has to
- * allocate a heap fragment where the result is stored
- * and update the the heap fragment pointer pointer
- * passed as third argument to point to it.
+ * context of the receiving process.
  *
- * If this function returns a reference, 'func' will
- * be called in the context of the receiver. However,
- * note that this might happen when the receiver is in
- * an exiting state. The caller of this function
+ * If 'reply' is non-zero, a response message '{Ref, Result}' is
+ * sent to the sender when 'func' has been called. 'Ref' is the
+ * reference returned by this function and 'Result' is the term
+ * returned by 'func'. If the return value of 'func' is not an
+ * immediate term, 'func' has to allocate a heap fragment where
+ * the result is stored and update the heap fragment pointer
+ * pointer passed as third argument to point to it.
+ *
+ * If 'reply' is zero, no automatic response is sent, the return value from
+ * 'func' is ignored and this function instead returns am_ok on success.
+ *
+ * If a receiving process with pid 'to' exists, 'func' will be
+ * called in its context. However, note that this might happen
+ * when the receiver is in an exiting state.
+ *
+ * If a reference is returned, the caller of this function
  * *unconditionally* has to enter a receive that match
  * on the returned reference in all clauses as next
  * receive; otherwise, bad things will happen!
@@ -969,27 +1034,24 @@ erts_proc_sig_send_sync_suspend(Process *c_p, Eterm to,
  *
  * @param[in]     reply         Non-zero if a reply is wanted.
  *
- * @param[in]     func          Function to execute in the
- *                              context of the receiver.
- *                              First argument will be a
- *                              pointer to the process struct
- *                              of the receiver process.
- *                              Second argument will be 'arg'
- *                              (see below). Third argument
- *                              will be a pointer to a pointer
- *                              to a heap fragment for storage
- *                              of result returned from 'func'
- *                              (i.e. an 'out' parameter).
+ * @param[in]     func          Function to execute in the context of the
+ *                              receiver. First argument will be a pointer to
+ *                              the process struct of the receiver process.
+ *                              Second argument will be 'arg' (see below). Third
+ *                              argument will be a pointer to number of
+ *                              reductions. In value is reductions left and
+ *                              absolute value out is consumed reduction where a
+ *                              negative value will force a yield. Fourth
+ *                              argument will be a pointer to a pointer to a
+ *                              heap fragment for storage of result returned
+ *                              from 'func' (i.e. an 'out' parameter).
  *
- * @param[in]     arg           Void pointer to argument
- *                              to pass as second argument
- *                              in call of 'func'.
+ * @param[in]     arg           Void pointer to argument to pass as second
+ *                              argument in call of 'func'.
  *
- * @returns                     If the request was sent,
- *                              an internal ordinary
- *                              reference; otherwise,
- *                              THE_NON_VALUE (non-existing
- *                              receiver).
+ * @returns                     If the request was sent, an internal ordinary
+ *                              reference or the atom ok; otherwise,
+ *                              THE_NON_VALUE (non-existing receiver).
  */
 Eterm
 erts_proc_sig_send_rpc_request(Process *c_p,
@@ -1009,7 +1071,7 @@ erts_proc_sig_send_rpc_request(Process *c_p,
  * term returned by 'func'. If the return value of
  * 'func' is not an immediate term, 'func' has to
  * allocate a heap fragment where the result is stored
- * and update the the heap fragment pointer pointer
+ * and update the heap fragment pointer pointer
  * passed as third argument to point to it.
  *
  * If this function returns a reference, 'func' will
@@ -1259,12 +1321,6 @@ erts_proc_sig_receive_helper(Process *c_p, int fcalls,
                              int neg_o_reds, ErtsMessage **msgpp,
                              int *get_outp);
 
-/*
- * CLEAN_SIGQ - Flush until middle queue is empty, i.e.
- *              the content of inner+middle queue equals
- *              the message queue.
- */
-#define ERTS_PROC_SIG_FLUSH_FLG_CLEAN_SIGQ          (1 << 0)
 /*
  * FROM_ALL   - Flush signals from all local senders (processes
  *              and ports).
@@ -1280,8 +1336,7 @@ erts_proc_sig_receive_helper(Process *c_p, int fcalls,
  * All erts_proc_sig_init_flush_signals() flags.
  */
 #define ERTS_PROC_SIG_FLUSH_FLGS                                \
-    (ERTS_PROC_SIG_FLUSH_FLG_CLEAN_SIGQ                         \
-     | ERTS_PROC_SIG_FLUSH_FLG_FROM_ALL                         \
+    (ERTS_PROC_SIG_FLUSH_FLG_FROM_ALL                           \
      | ERTS_PROC_SIG_FLUSH_FLG_FROM_ID)
 
 /**
@@ -1334,19 +1389,46 @@ ERTS_GLB_INLINE Sint erts_proc_sig_fetch(Process *p);
 
 /**
  *
- * @brief Get amount of messages in private queues
+ * @brief Get amount of signals in private queues
  *
  * @param[in]   c_p             Pointer to process struct of
  *                              currently executing process.
  *
- * @returns                     Amount of message signals in
+ * @param[in]   max_nmsigs      Maximum amount of signals to
+ *                              traverse in order to calculate
+ *                              the result. If this limit is
+ *                              reached the operation is aborted
+ *                              and the current calculated
+ *                              result is returned as a negative
+ *                              amount. If a negative number
+ *                              is given, an unlimited amount
+ *                              of signals will be traversed in
+ *                              order to calculate the result.
+ *
+ *
+ * @param[in]   max_nmsigs      Maximum amount of non-message
+ *                              signals to traverse in order
+ *                              to calculate the result. If
+ *                              this limit is reached the
+ *                              operation is aborted and the
+ *                              current calculated result
+ *                              is returned as a negative
+ *                              amount. If a negative number
+ *                              is given, an unlimited amount
+ *                              of non-message signals will
+ *                              be traversed in order to
+ *                              calculate the result.
+ *
+ * @returns                     Amount of signals in
  *                              inner plus middle signal
  *                              queues after fetch completed
- *                              (NOT the message queue
- *                              length).
+ *                              (this is NOT the message queue
+ *                              length). Negative amount
+ *                              if the operation was aborted
+ *                              (see above).
  */
-Sint
-erts_proc_sig_privqs_len(Process *c_p);
+ERTS_GLB_INLINE Sint
+erts_proc_sig_privqs_len(Process *c_p, Sint max_sigs, Sint max_nmsigs);
 
 /**
  * @brief Enqueue a sequence of signals on an in signal queue of
@@ -1758,9 +1840,8 @@ extern Process *erts_dirty_process_signal_handler_max;
 void erts_proc_sig_fetch__(Process *proc,
                            ErtsSignalInQueueBufferArray* buffers,
                            int need_unget_buffers);
-Sint erts_proc_sig_fetch_msgq_len_offs__(Process *proc,
-                                         ErtsSignalInQueueBufferArray* buffers,
-                                         int need_unget_buffers);
+ERTS_GLB_INLINE void erts_chk_sys_mon_long_msgq_on(Process *proc);
+ERTS_GLB_INLINE void erts_chk_sys_mon_long_msgq_off(Process *proc);
 ERTS_GLB_INLINE int erts_msgq_eq_recv_mark_id__(Eterm term1, Eterm term2);
 ERTS_GLB_INLINE void erts_msgq_recv_marker_set_save__(Process *c_p,
 				 ErtsRecvMarkerBlock *blkp,
@@ -1847,11 +1928,32 @@ erts_proc_sig_queue_unget_buffers(ErtsSignalInQueueBufferArray* buffers,
     }
 }
 
+ERTS_GLB_INLINE void
+erts_chk_sys_mon_long_msgq_on(Process *proc)
+{
+    if (((proc->sig_qs.flags & (FS_MON_MSGQ_LEN|FS_MON_MSGQ_LEN_LONG))
+         == FS_MON_MSGQ_LEN)
+        && proc->sig_qs.mq_len >= erts_system_monitor_long_msgq_on) {
+        proc->sig_qs.flags |= FS_MON_MSGQ_LEN_LONG;
+        monitor_generic(proc, am_long_message_queue, am_true);
+    }
+}
+
+ERTS_GLB_INLINE void
+erts_chk_sys_mon_long_msgq_off(Process *proc)
+{
+    if (((proc->sig_qs.flags & (FS_MON_MSGQ_LEN|FS_MON_MSGQ_LEN_LONG))
+         == (FS_MON_MSGQ_LEN|FS_MON_MSGQ_LEN_LONG))
+        && proc->sig_qs.mq_len <= erts_system_monitor_long_msgq_off) {
+        proc->sig_qs.flags &= ~FS_MON_MSGQ_LEN_LONG;
+        monitor_generic(proc, am_long_message_queue, am_false);
+    }
+}
+
 ERTS_GLB_INLINE Sint
 erts_proc_sig_fetch(Process *proc)
 {
-    Sint res = 0;
-    ErtsSignal *sig;
+    Sint res;
     ErtsSignalInQueueBufferArray* buffers;
     int need_unget_buffers;
     ERTS_LC_ASSERT((erts_proc_lc_my_proc_locks(proc)
@@ -1873,38 +1975,43 @@ erts_proc_sig_fetch(Process *proc)
 
     buffers = erts_proc_sig_queue_get_buffers(proc, &need_unget_buffers);
 
-    sig = (ErtsSignal *) proc->sig_inq.first;
-
-    if (!sig) {
-        if (buffers)
-            goto fetch;
-    }
-    else if (ERTS_UNLIKELY(sig->common.tag
-                           == ERTS_PROC_SIG_MSGQ_LEN_OFFS_MARK)) {
-        res = erts_proc_sig_fetch_msgq_len_offs__(proc, buffers,
-                                                  need_unget_buffers);
-    }
-    else {
-    fetch:
+    if (buffers || proc->sig_inq.first)
         erts_proc_sig_fetch__(proc, buffers, need_unget_buffers);
-    }
-
-    res += proc->sig_qs.len;
+    res = proc->sig_qs.mq_len;
 
     ERTS_HDBG_CHECK_SIGNAL_PRIV_QUEUE(proc, !0);
 
-#ifdef ERTS_PROC_SIG_HARD_DEBUG_SIGQ_MSG_LEN
-    {
-        Sint len = 0;
-        ERTS_FOREACH_SIG_PRIVQS(
-            proc, mp,
-            {
-                if (ERTS_SIG_IS_MSG(mp))
-                    len++;
-            });
-        ERTS_ASSERT(res == len);
+    return res;
+}
+
+ERTS_GLB_INLINE Sint
+erts_proc_sig_privqs_len(Process *c_p, Sint max_sigs, Sint max_nmsigs)
+{
+    Sint res = c_p->sig_qs.mq_len;
+    int no_nmsigs = 0;
+    ErtsMessage **nmsigpp;
+
+    ERTS_HDBG_PRIVQ_LEN(c_p);
+
+    if (max_sigs < 0)
+        max_sigs = ERTS_SWORD_MAX; /* Check all... */
+    if (res > max_sigs)
+        return -res;
+
+    nmsigpp = c_p->sig_qs.nmsigs.next;
+    if (max_nmsigs < 0)
+        max_nmsigs = ERTS_SWORD_MAX; /* Check all... */
+    res += c_p->sig_qs.mlenoffs;
+    while (nmsigpp) {
+        ErtsNonMsgSignal *nmsigp = (ErtsNonMsgSignal *) *nmsigpp;
+        ASSERT(nmsigp);
+        res += nmsigp->mlenoffs + 1;
+        if (res > max_sigs)
+            return -res;
+        if (++no_nmsigs > max_nmsigs)
+            return -res; /* Abort; to many non-message signals... */
+        nmsigpp = nmsigp->specific.next;
     }
-#endif
 
     return res;
 }
@@ -1953,25 +2060,6 @@ erts_proc_notify_new_message(Process *p, ErtsProcLocks locks)
 	}								\
     } while (0)
 
-#undef ERTS_PROC_SIG_RECV_MARK_CLEAR_OLD_MARK__
-#ifdef ERTS_SUPPORT_OLD_RECV_MARK_INSTRS
-
-#define ERTS_PROC_SIG_RECV_MARK_CLEAR_OLD_MARK__(BLKP)			\
-    do {								\
-	if ((BLKP)->old_recv_marker_ix >= 0) {				\
-	    int ix__ = (BLKP)->old_recv_marker_ix;			\
-	    ASSERT((BLKP)->ref[ix__] == erts_old_recv_marker_id);	\
-	    ASSERT((BLKP)->marker[ix__].in_sigq);			\
-	    ASSERT(!(BLKP)->marker[ix__].set_save);			\
-	    (BLKP)->unused++;						\
-	    (BLKP)->ref[ix__] = am_undefined;				\
-	    (BLKP)->marker[ix__].pass = ERTS_RECV_MARKER_PASS_MAX;	\
-	    (BLKP)->old_recv_marker_ix = -1;				\
-	}								\
-    } while (0)
-
-#endif
-
 ERTS_GLB_INLINE int
 erts_msgq_eq_recv_mark_id__(Eterm term1, Eterm term2)
 {
@@ -2053,13 +2141,6 @@ erts_msgq_recv_marker_clear(Process *c_p, Eterm id)
 
     if (!blkp)
 	return;
-    
-#ifdef ERTS_SUPPORT_OLD_RECV_MARK_INSTRS
-    if (id == erts_old_recv_marker_id) {
-	ERTS_PROC_SIG_RECV_MARK_CLEAR_OLD_MARK__(blkp);
-	return;
-    }
-#endif
 
     for (ix = 0; ix < ERTS_RECV_MARKER_BLOCK_SIZE; ix++) {
 	if (erts_msgq_eq_recv_mark_id__(blkp->ref[ix], id)) {
@@ -2089,9 +2170,6 @@ ERTS_GLB_INLINE void erts_msgq_recv_marker_bind(Process *c_p,
 						Eterm bind_id)
 {
     ASSERT(!(c_p->sig_qs.flags & FS_HANDLING_SIGS));
-#ifdef ERTS_SUPPORT_OLD_RECV_MARK_INSTRS
-    ASSERT(bind_id != erts_old_recv_marker_id);
-#endif
 
     if (is_small(insert_id) || is_big(insert_id)) {
 	ErtsRecvMarkerBlock *blkp = c_p->sig_qs.recv_mrk_blk;
@@ -2120,12 +2198,6 @@ erts_msgq_recv_marker_insert_bind(Process *c_p, Eterm id)
 {
     ASSERT(!(c_p->sig_qs.flags & FS_HANDLING_SIGS));
     if (is_internal_ref(id)) {
-#ifdef ERTS_SUPPORT_OLD_RECV_MARK_INSTRS
-	ErtsRecvMarkerBlock *blkp = c_p->sig_qs.recv_mrk_blk;
-	if (blkp && erts_old_recv_marker_id == id)
-	    ERTS_PROC_SIG_RECV_MARK_CLEAR_OLD_MARK__(blkp);
-#endif
-
         erts_proc_sig_queue_lock(c_p);
 	erts_proc_sig_fetch(c_p);
 	erts_proc_unlock(c_p, ERTS_PROC_LOCK_MSGQ);
@@ -2171,7 +2243,9 @@ erts_msgq_unlink_msg(Process *c_p, ErtsMessage *msgp)
     ASSERT(!(c_p->sig_qs.flags & FS_HANDLING_SIGS));
     ERTS_HDBG_CHECK_SIGNAL_PRIV_QUEUE__(c_p, 0, "before");
     *c_p->sig_qs.save = sigp;
-    c_p->sig_qs.len--;
+    c_p->sig_qs.mq_len--;
+    ASSERT(c_p->sig_qs.mq_len >= 0);
+    erts_chk_sys_mon_long_msgq_off(c_p);
     if (sigp && ERTS_SIG_IS_RECV_MARKER(sigp)) {
         ErtsMessage **sigpp = c_p->sig_qs.save;
         ((ErtsRecvMarker *) sigp)->prev_next = sigpp;
@@ -2190,9 +2264,6 @@ erts_msgq_set_save_first(Process *c_p)
     ASSERT(!(c_p->sig_qs.flags & FS_HANDLING_SIGS));
     if (blkp) {
 	ERTS_PROC_SIG_RECV_MARK_CLEAR_PENDING_SET_SAVE__(blkp);
-#ifdef ERTS_SUPPORT_OLD_RECV_MARK_INSTRS
-	ERTS_PROC_SIG_RECV_MARK_CLEAR_OLD_MARK__(blkp);
-#endif
     }    
 
     /*
@@ -2213,7 +2284,9 @@ erts_msgq_unlink_msg_set_save_first(Process *c_p, ErtsMessage *msgp)
     ASSERT(!(c_p->sig_qs.flags & FS_HANDLING_SIGS));
     ERTS_HDBG_CHECK_SIGNAL_PRIV_QUEUE__(c_p, 0, "before");
     *c_p->sig_qs.save = sigp;
-    c_p->sig_qs.len--;
+    c_p->sig_qs.mq_len--;
+    ASSERT(c_p->sig_qs.mq_len >= 0);
+    erts_chk_sys_mon_long_msgq_off(c_p);
     if (!sigp)
         c_p->sig_qs.last = c_p->sig_qs.save;
     else if (ERTS_SIG_IS_RECV_MARKER(sigp))
