@@ -1,7 +1,7 @@
 %%
 %% %CopyrightBegin%
 %%
-%% Copyright Ericsson AB 2005-2022. All Rights Reserved.
+%% Copyright Ericsson AB 2005-2024. All Rights Reserved.
 %%
 %% Licensed under the Apache License, Version 2.0 (the "License");
 %% you may not use this file except in compliance with the License.
@@ -28,11 +28,11 @@
 -export([display/1, display_huge/0, display_string/1,
 	 erl_bif_types/1,guard_bifs_in_erl_bif_types/1,
 	 shadow_comments/1,list_to_utf8_atom/1,
-	 specs/1,improper_bif_stubs/1,auto_imports/1,
+	 specs/1,improper_bif_stubs/1,
 	 t_list_to_existing_atom/1,os_env/1,otp_7526/1,
 	 t_binary_to_atom/1,t_binary_to_existing_atom/1,
 	 t_atom_to_binary/1,min_max/1, erlang_halt/1,
-         erl_crash_dump_bytes/1,
+         halt_flush_timeout/1, erl_crash_dump_bytes/1,
 	 is_builtin/1, error_stacktrace/1,
 	 error_stacktrace_during_call_trace/1,
          group_leader_prio/1, group_leader_prio_dirty/1,
@@ -43,7 +43,8 @@
          verify_middle_queue_save/1,
          test_length/1,
          fixed_apply_badarg/1,
-         external_fun_apply3/1]).
+         external_fun_apply3/1,
+         node_1/1]).
 
 suite() ->
     [{ct_hooks,[ts_install_cth]},
@@ -51,17 +52,17 @@ suite() ->
 
 all() -> 
     [erl_bif_types, guard_bifs_in_erl_bif_types, shadow_comments,
-     specs, improper_bif_stubs, auto_imports,
+     specs, improper_bif_stubs,
      t_list_to_existing_atom, os_env, otp_7526,
      display, display_string, list_to_utf8_atom,
      t_atom_to_binary, t_binary_to_atom, t_binary_to_existing_atom,
-     erl_crash_dump_bytes, min_max, erlang_halt, is_builtin,
-     error_stacktrace, error_stacktrace_during_call_trace,
+     erl_crash_dump_bytes, min_max, erlang_halt, halt_flush_timeout,
+     is_builtin, error_stacktrace, error_stacktrace_during_call_trace,
      group_leader_prio, group_leader_prio_dirty,
      is_process_alive, is_process_alive_signal_from,
      process_info_blast, os_env_case_sensitivity,
      verify_middle_queue_save, test_length,fixed_apply_badarg,
-     external_fun_apply3].
+     external_fun_apply3, node_1].
 
 init_per_testcase(guard_bifs_in_erl_bif_types, Config) when is_list(Config) ->
     skip_missing_erl_bif_types(Config);
@@ -324,38 +325,14 @@ improper_bif_stubs(_) ->
     [check_stub(MFA, Body) || {MFA,Body} <- sofs:to_external(FuncRel)],
     ok.
 
-auto_imports(_Config) ->
-    Path = get_code_path(),
-    {erlang,Abstr} = extract_abstract(erlang, Path),
-    SpecFuns = [Name || {attribute,_,spec,{Name,_}} <- Abstr],
-    auto_imports(SpecFuns, 0).
-
-auto_imports([{F,A}|T], Errors) ->
-    case erl_internal:bif(F, A) of
-	false ->
-	    io:format("~p/~p: not auto-imported, but spec claims it "
-		      "is auto-imported", [F,A]),
-	    auto_imports(T, Errors+1);
-	true ->
-	    auto_imports(T, Errors)
-    end;
-auto_imports([{erlang,F,A}|T], Errors) ->
-    case erl_internal:bif(F, A) of
-	false ->
-	    auto_imports(T, Errors);
-	true ->
-	    io:format("~p/~p: auto-imported, but "
-		      "spec claims it is *not* auto-imported", [F,A]),
-	    auto_imports(T, Errors+1)
-    end;
-auto_imports([], 0) ->
-    ok;
-auto_imports([], Errors) ->
-    ct:fail({Errors,inconsistencies}).
-
 extract_functions(M, Abstr) ->
     [{{M,F,A},Body} || {function,_,F,A,Body} <- Abstr].
 
+check_stub({_,F,2}, _B) when F =:= min; F =:= max ->
+    %% In Erlang/OTP 26, min/2 and max/2 are guard BIFs. For backward
+    %% compatibility with code compiled with an earlier version, the
+    %% Erlang implementation of them is kept.
+    ok;
 check_stub({_,F,A}, B) ->
     try
 	[{clause,_,Args,[],Body}] = B,
@@ -724,33 +701,147 @@ fail_atom_to_binary(Term) ->
     end.
 
 
-min_max(Config) when is_list(Config) ->	
-    a = erlang:min(id(a), a),
-    a = erlang:min(id(a), b),
-    a = erlang:min(id(b), a),
-    b = erlang:min(id(b), b),
-    a = erlang:max(id(a), a),
-    b = erlang:max(id(a), b),
-    b = erlang:max(id(b), a),
-    b = erlang:max(id(b), b),
-
-    42.0 = erlang:min(42.0, 42),
-    42.0 = erlang:max(42.0, 42),
-    %% And now (R14) they are also autoimported!
+min_max(Config) when is_list(Config) ->
+    Self = self(),
+    Port = hd(erlang:ports()),
+    Ref = make_ref(),
     a = min(id(a), a),
     a = min(id(a), b),
     a = min(id(b), a),
     b = min(id(b), b),
+    Ref = min(id(Self), id(Ref)),
+
+    -3 = min(id(5), -3),
+    -3 = min(-3, id(5)),
+    -3 = min(0, id(-3)),
+    -3 = min(id(-3), 0),
+    0 = min(0, id(17)),
+
     a = max(id(a), a),
     b = max(id(a), b),
     b = max(id(b), a),
     b = max(id(b), b),
+    Self = max(id(Self), id(Ref)),
+
+    5 = max(id(5), -3),
+    5 = max(-3, id(5)),
+    0 = max(0, id(-3)),
+    0 = max(id(-3), 0),
+    17 = max(0, id(17)),
+
+    %% Return the first argument when arguments are equal.
+    42.0 = min(id(42.0), 42),
+    42.0 = max(id(42.0), 42),
+
+    Min = id(min),
+    Max = id(max),
+
+    "abc" = erlang:Min("abc", "def"),
+    <<"def">> = erlang:Max(<<"abc">>, <<"def">>),
+
+    %% Make sure that the JIT doesn't do any unsafe optimizations.
+    {0, 0} = min_max_zero(0),
+    {-7, 0} = min_max_zero(-7),
+    {0, 555} = min_max_zero(555),
+    {0, 1 bsl 64} = min_max_zero(1 bsl 64),
+    {-1 bsl 64, 0} = min_max_zero(-1 bsl 64),
+
+    {-99, 23} = do_min_max(-99, 23),
+    {-10, 0} = do_min_max(0, -10),
+    {0, 77} = do_min_max(77, 0),
+    {1, 2} = do_min_max(1, 2),
+    {42, 99} = do_min_max(99, 42),
+    {100, 1 bsl 64} = do_min_max(100, 1 bsl 64),
+    {-1 bsl 64, 77} = do_min_max(77, -1 bsl 64),
+    {-1 bsl 64, 1 bsl 64} = do_min_max(1 bsl 64, -1 bsl 64),
+    {42.0, 43} = do_min_max(42.0, 43),
+    {42.0, 50.0} = do_min_max(42.0, 50.0),
+    {42.0, 42.0} = do_min_max(42.0, id(40.0 + 2.0)),
+    {{1,2}, {a,b}} = do_min_max({id(a), id(b)}, {id(1), id(2)}),
+    {{a,b}, [a,b]} = do_min_max({a,id(b)}, [a,id(b)]),
+    {{1.0,b}, {1.0,b}} = do_min_max({id(1.0), id(b)}, {id(1), id(b)}),
+    {{7,b}, {7,b}} = do_min_max({id(7), id(b)}, {id(7.0), id(b)}),
+
+    {42,Self} = do_min_max(42, Self),
+    {42,Self} = do_min_max(Self, 42),
+    {42,Port} = do_min_max(42, Port),
+    {42,Port} = do_min_max(Port, 42),
 
-    42.0 = min(42.0, 42),
-    42.0 = max(42.0, 42),
     ok.
 
+min_max_zero(A0) ->
+    Result = {min(A0, 0), max(A0, 0)},
+    Result = {min(0, A0), max(0, A0)},
+    A = id(A0),
+    Result = {min(A, 0), max(A, 0)},
+    Result = {min(0, A), max(0, A)}.
 
+do_min_max(A0, B0) ->
+    Result = {min(A0, B0), max(A0, B0)},
+
+    A0 = min(id(A0), A0),
+    A0 = max(id(A0), A0),
+    B0 = min(id(B0), B0),
+    B0 = max(id(B0), B0),
+
+    A = id(A0),
+    B = id(B0),
+    Result = {min(A, B), max(A, B)},
+
+    if
+        is_integer(A), is_atom(node(B)) orelse is_integer(B) ->
+            _ = id(0),
+            Result = {min(A, B),max(A, B)};
+        is_atom(node(A)) orelse is_integer(A), is_integer(B) ->
+            _ = id(0),
+            Result = {min(A, B),max(A, B)};
+        true ->
+            ok
+    end,
+
+    if
+        is_integer(A), 0 =< A, A =< 1000, is_atom(node(B)) orelse is_integer(B) ->
+            _ = id(0),
+            Result = {min(A, B),max(A, B)};
+        is_atom(node(A)) orelse is_integer(A), is_integer(B), 0 =< B, B =< 1000 ->
+            _ = id(0),
+            Result = {min(A, B),max(A, B)};
+        true ->
+            ok
+    end,
+
+    Result = do_min_max_1(1, 2, 3, 4, 5, A, B).
+
+do_min_max_1(_, _, _, _, _, A, B) ->
+    if
+        is_integer(A), 0 =< A, A < 16#1_0000,
+        is_integer(B), 0 =< B, B < 16#1_0000 ->
+            Result = {min(A, B),max(A, B)},
+            Result = {min(B, A),max(B, A)},
+            _ = id(0),
+            Result = {min(A, B),max(A, B)},
+            Result = {min(B, A),max(B, A)};
+        is_integer(A), is_integer(B) ->
+            Result = {min(A, B),max(A, B)},
+            Result = {min(B, A),max(B, A)},
+            _ = id(0),
+            Result = {min(A, B),max(A, B)},
+            Result = {min(B, A),max(B, A)};
+        is_float(A), is_float(B) ->
+            Result = {min(A, B),max(A, B)},
+            Result = {min(B, A),max(B, A)},
+            _ = id(0),
+            Result = {min(A, B),max(A, B)},
+            Result = {min(B, A),max(B, A)};
+        is_number(A), is_number(B) ->
+            Result = {min(A, B),max(A, B)},
+            _ = id(0),
+            Result = {min(A, B),max(A, B)};
+        true ->
+            Result = {min(A, B),max(A, B)},
+            _ = id(0),
+            Result = {min(A, B),max(A, B)}
+    end.
 
 erlang_halt(Config) when is_list(Config) ->
     try erlang:halt(undefined) of
@@ -817,6 +908,167 @@ erlang_halt(Config) when is_list(Config) ->
             ok
     end.
 
+
+halt_flush_timeout(Config) when is_list(Config) ->
+    ct:timetrap({minutes, 5}),
+    halt_flush_timeout_test(false, true),
+    halt_flush_timeout_test(true, true),
+    halt_flush_timeout_test(true, false).
+
+halt_flush_timeout_test(HaltCmd, HaltOpt) ->
+    halt_flush_timeout_test_try(HaltCmd, HaltOpt, 1).
+
+halt_flush_timeout_test_try(HaltCmd, HaltOpt, N) ->
+    case {HaltCmd, HaltOpt} of
+        {false, true} ->
+            ct:log("Test no ~p with flush_timeout option and "
+                   "default command line~n", [N]);
+        {true, true} ->
+            ct:log("Test no ~p with flush_timeout option and "
+                   "command line arg~n", [N]);
+        {true, false} ->
+            ct:log("Test no ~p with command line arg~n", [N])
+    end,
+    {ok, Peer, Node} = ?CT_PEER(),
+    try
+        ok = halt_flush_timeout_test_run(Node, HaltCmd, HaltOpt),
+        peer:stop(Peer),
+        ok
+    catch
+        Class:Reason:Stack ->
+            ct:log("Failed with ~p reason: ~p~n"
+                   "at ~p~n", [Class, Reason, Stack]),
+            peer:stop(Peer),
+            if N == 5 ->
+                    erlang:raise(Class, Reason, Stack);
+               true ->
+                    halt_flush_timeout_test_try(HaltCmd, HaltOpt, N+1)
+            end
+    end.
+
+halt_flush_timeout_test_run(BNode, HaltCmd, HaltOpt) ->
+    {ok, HNodePort, HNode} = start_halting_node(if HaltCmd == true -> "+zhft 1500";
+                                                   true -> ""
+                                                end),
+    ok = erpc:call(BNode,
+                   fun () ->
+                           pong = net_adm:ping(HNode),
+                           erts_debug:set_internal_state(available_internal_state, true),
+                           ok
+                   end),
+    erpc:cast(BNode,
+              fun () ->
+                      erts_debug:set_internal_state(block, 60*1000)
+              end),
+    wait_until(fun () ->
+                       try
+                           BNode = erpc:call(BNode, erlang, node, [], 1000),
+                           false
+                       catch
+                           error:{erpc,timeout} ->
+                               true
+                       end
+               end),
+    Data = lists:seq(1,1000),
+    SendData = fun SendData() ->
+                       {net_kernel, BNode} ! Data,
+                       SendData()
+               end,
+    Pid1 = spawn(HNode, SendData),
+    Pid2 = spawn(HNode, SendData),
+    IsBlocked = fun (Pid) ->
+                        fun () ->
+                                case erpc:call(node(Pid), erlang, process_info, [Pid, status]) of
+                                    {status, suspended} -> true;
+                                    _Val -> false
+                                end
+                        end
+                end,
+    wait_until(IsBlocked(Pid1)),
+    wait_until(IsBlocked(Pid2)),
+    Start = erlang:monotonic_time(),
+    try
+        erpc:call(HNode,
+                  fun () ->
+                          if HaltOpt == true -> halt(0, [{flush, true},{flush_timeout, 1000}]);
+                             true -> halt()
+                          end
+                  end),
+        error(unexpected_return)
+    catch
+        error:{erpc,noconnection} ->
+            ok
+    end,
+    ExitStatus = await_halting_node_exit(HNodePort),
+    End = erlang:monotonic_time(),
+    ct:log("ExitStatus=~p~n", [ExitStatus]),
+    case erlang:convert_time_unit(End - Start, native, millisecond) of
+        HaltTime when HaltOpt == true, 1000 =< HaltTime, HaltTime < 1500 ->
+            ok;
+        HaltTime when HaltOpt /= true, 1500 =< HaltTime, HaltTime < 2000 ->
+            ok;
+        HaltTime ->
+            error({unexpected_halt_time, HaltTime})
+    end,
+    ExitStatus = 255, %% Exit status when timing out...
+    ok.
+
+start_halting_node(Args) ->
+    Name = "halting_node-"
+        ++ integer_to_list(erlang:system_time(second)) ++ "-"
+        ++ integer_to_list(erlang:unique_integer([positive])),
+    HostSuffix = lists:dropwhile(fun ($@) -> false; (_) -> true end,
+				 atom_to_list(node())),
+    Node = list_to_atom(Name ++ HostSuffix),
+    Pa = filename:dirname(code:which(?MODULE)),
+    Prog = case catch init:get_argument(progname) of
+	       {ok,[[P]]} -> P;
+	       _ -> exit(no_progname_argument_found)
+	   end,
+    NameSw = case net_kernel:longnames() of
+		 false -> "-sname ";
+		 true -> "-name ";
+		 _ -> exit(not_distributed_node)
+	     end,
+    {ok, Pwd} = file:get_cwd(),
+    CmdLine =
+        Prog ++ " -noinput -noshell " ++ Args
+	++ " " ++ NameSw ++ " " ++ Name ++ " "
+	++ "-pa " ++ Pa ++ " "
+	++ "-env ERL_CRASH_DUMP " ++ Pwd ++ "/erl_crash_dump." ++ Name ++ " "
+	++ "-setcookie " ++ atom_to_list(erlang:get_cookie()),
+    ct:log("Starting node ~p: ~s~n", [Node, CmdLine]),
+    case open_port({spawn, CmdLine}, [exit_status]) of
+	Port when is_port(Port) ->
+            PingNode = fun PingNode(_PNode, 0) ->
+                               pang;
+                           PingNode(PNode, N) ->
+                               case net_adm:ping(PNode) of
+                                   pong ->
+                                       pong;
+                                   _ ->
+                                       receive after 100 -> ok end,
+                                       PingNode(PNode, N-1)
+                               end
+                       end,
+            case PingNode(Node, 50) of
+		pong ->
+                    {ok, Port, Node};
+		Other ->
+                    error({failed_to_start_node, Node, Other})
+	    end;
+	Error ->
+            error({failed_to_start_node, Node, Error})
+    end.
+
+await_halting_node_exit(Port) ->
+    receive
+        {Port, {data, _}} ->
+            await_halting_node_exit(Port);
+        {Port, {exit_status, ExitStatus}} ->
+            ExitStatus
+    end.
+
 add_asan_opt(Opt) ->
     case test_server:is_asan() of
 	true ->
@@ -1445,8 +1697,84 @@ external_fun_apply3(_Config) ->
 
     ok.
 
+node_1(_Config) ->
+    {ok, Peer, Node} = ?CT_PEER(),
+
+    local_node(self()),
+    LocalPort = lists:last(erlang:ports()),
+    local_node(LocalPort),
+    local_node(make_ref()),
+
+    external_node(erpc:call(Node, erlang, self, []), Node),
+    ExtPort = hd(erpc:call(Node, erlang, ports, [])),
+    external_node(ExtPort, Node),
+    external_node(erpc:call(Node, erlang, make_ref, []), Node),
+
+    node_error(a),
+    node_error(42),
+    node_error({a,b,c}),
+    node_error({tag,self()}),
+    node_error([self()]),
+    node_error(1 bsl 133),
+    node_error(#{}),
+    node_error(#{id(a) => b}),
+    node_error(<<"binary">>),
+
+    peer:stop(Peer),
+    ok.
+
+local_node(E) ->
+    test_node(E, node()).
+
+external_node(E, Node) ->
+    test_node(E, Node).
+
+test_node(E0, Node) ->
+    true = node(id(E0)) =:= Node,
+    E = id(E0),
+    if
+        node(E) =:= Node ->
+            ok
+    end,
+    test_node_2(id(E), Node).
+
+test_node_2(E, Node) when is_pid(E); is_port(E); is_reference(E) ->
+    true = node(E) =:= Node,
+    if
+        node(E) =:= Node ->
+            ok
+    end,
+    test_node_3(id(E), Node),
+    ok.
+
+test_node_3(E, Node) when is_pid(E) ->
+    true = node(E) =:= Node;
+test_node_3(E, Node) when is_port(E) ->
+    true = node(E) =:= Node;
+test_node_3(E, Node) when is_reference(E) ->
+    true = node(E) =:= Node.
+
+node_error(E0) ->
+    E = id(E0),
+    {'EXIT',{badarg,[{erlang,node,[E],_}|_]}} = catch node(E),
+    if
+        node(E) ->
+            ct:fail(should_fail);
+        true ->
+            ok
+    end.
+
 %% helpers
 
+wait_until(Fun) ->
+    case Fun() of
+        true ->
+            ok;
+        _ ->
+            receive after 100 -> ok end,
+            wait_until(Fun)
+    end.
+
 busy_wait_go() ->
     receive
         {go, Info} ->
