@@ -1,7 +1,7 @@
 %%
 %% %CopyrightBegin%
 %% 
-%% Copyright Ericsson AB 1997-2021. All Rights Reserved.
+%% Copyright Ericsson AB 1997-2023. All Rights Reserved.
 %% 
 %% Licensed under the Apache License, Version 2.0 (the "License");
 %% you may not use this file except in compliance with the License.
@@ -32,6 +32,7 @@
     grow_stack/1,
     grow_stack_heap/1,
     max_heap_size/1,
+    max_heap_size_large_hfrag/1,
     minor_major_gc_option_async/1,
     minor_major_gc_option_self/1,
     gc_signal_order/1,
@@ -44,6 +45,7 @@ suite() ->
 
 all() -> 
     [grow_heap, grow_stack, grow_stack_heap, max_heap_size,
+     max_heap_size_large_hfrag,
     minor_major_gc_option_self,
     minor_major_gc_option_async, gc_signal_order, gc_dirty_exec_proc,
     alias_signals_in_gc].
@@ -206,6 +208,30 @@ long_receive() ->
             ok
     end.
 
+%% Trigger gc-after-bif with a large heap fragment, which provoked some bugs.
+max_heap_size_large_hfrag(Config) ->
+    max_heap_size_large_hfrag_do(),
+    %% Repeat with major GC
+    process_flag(fullsweep_after, 0),
+    max_heap_size_large_hfrag_do(),
+    ok.
+
+max_heap_size_large_hfrag_do() ->
+    {Pid, Ref} =
+        spawn_opt(
+          fun Fun() ->
+                  erlang:make_tuple(2000, []),
+                  Fun()
+          end,
+          [monitor, {max_heap_size, 1000}]),
+    %% Verify that max heap was triggered
+    {'DOWN', Ref, process, Pid, killed} =
+        receive M -> M
+        after 5000 ->
+                ct:fail({process_did_not_die, Pid, erlang:process_info(Pid)})
+        end,
+    ok.
+
 minor_major_gc_option_self(_Config) ->
     %% Try as major, the test process will self-trigger GC
     check_gc_tracing_around(
