@@ -1,7 +1,7 @@
 %%
 %% %CopyrightBegin%
 %% 
-%% Copyright Ericsson AB 1999-2023. All Rights Reserved.
+%% Copyright Ericsson AB 1999-2024. All Rights Reserved.
 %% 
 %% Licensed under the Apache License, Version 2.0 (the "License");
 %% you may not use this file except in compliance with the License.
@@ -25,7 +25,8 @@
 -export([all/0, suite/0,
          bsl_bsr/1,logical/1,t_not/1,relop_simple/1,relop/1,
          complex_relop/1,unsafe_fusing/1,
-         range_tests/1,typed_relop/1]).
+         range_tests/1,combined_relops/1,typed_relop/1,
+         term_equivalence/1]).
 
 -import(lists, [foldl/3,flatmap/2]).
 
@@ -35,7 +36,8 @@ suite() ->
 
 all() ->
     [bsl_bsr, logical, t_not, relop_simple, relop,
-     complex_relop, unsafe_fusing, range_tests, typed_relop].
+     complex_relop, unsafe_fusing, range_tests,
+     combined_relops, typed_relop, term_equivalence].
 
 %% Test the bsl and bsr operators.
 bsl_bsr(Config) when is_list(Config) ->
@@ -145,8 +147,8 @@ relop_simple(Config) when is_list(Config) ->
     F2 = float(Big2),
     T1 = erlang:make_tuple(3,87),
     T2 = erlang:make_tuple(3,87),
-    Terms = [-F2,Big2,-F1,-Big1,-33,-33.0,0,0.0,42,42.0,Big1,F1,Big2,F2,a,b,
-             {T1,a},{T2,b},[T1,Big1],[T2,Big2]],
+    Terms = [-F2,Big2,-F1,-Big1,-33,-33.0,0,0.0,-0.0,42,42.0,Big1,F1,Big2,F2,
+             a,b,{T1,a},{T2,b},[T1,Big1],[T2,Big2]],
 
     Combos = [{V1,V2} || V1 <- Terms, V2 <- Terms],
 
@@ -180,6 +182,8 @@ relop_simple_do(V1,V2) ->
     ID = not (V1 =/= V2),
     ID = not (V2 =/= V1),
 
+    implies(ID, V1 == V2),
+
     EQ = V1 == V2,
     EQ = V2 == V1,
     EQ = not (V1 /= V2),
@@ -192,6 +196,9 @@ relop_simple_do(V1,V2) ->
         {false, false, false, true,  +1} -> ok
     end.
 
+implies(false, _) -> ok;
+implies(true, true) -> ok.
+
 %% Emulate internal "cmp"
 cmp_emu(A,B) when is_tuple(A), is_tuple(B) ->
     SA = size(A),
@@ -293,7 +300,12 @@ relop(Config) when is_list(Config) ->
     Big2 = 19738924729729787487784874,
     F1 = float(Big1),
     F2 = float(Big2),
-    Vs0 = [a,b,-33,-33.0,0,0.0,42,42.0,Big1,Big2,F1,F2],
+    Bin = <<"abc">>,
+    BitString = <<0:7>>,
+    Map = #{a => b},
+    EmptyMap = #{},
+    Vs0 = [a,b,-33,-33.0,0,0.0,-0,0,42,42.0,Big1,Big2,F1,F2,
+           Bin,BitString,Map,EmptyMap,[16#1234_5678_abcd]],
     Vs = [unvalue(V) || V <- Vs0],
     Ops = ['==', '/=', '=:=', '=/=', '<', '=<', '>', '>='],
     binop(Ops, Vs).
@@ -302,7 +314,13 @@ relop(Config) when is_list(Config) ->
 complex_relop(Config) when is_list(Config) ->
     Big = 99678557475484872464269855544643333,
     Float = float(Big),
-    Vs0 = [an_atom,42.0,42,Big,Float],
+    Bin = <<"abc">>,
+    BitString = <<0:7>>,
+    EmptyBitString = <<>>,
+    Map = #{a => b},
+    EmptyMap = #{},
+    Vs0 = [an_atom,42.0,42,0.0,-0.0,Big,Float,Bin,BitString,
+           EmptyBitString,Map,EmptyMap],
     Vs = flatmap(fun(X) -> [unvalue({X}),unvalue([X])] end, Vs0),
     Ops = ['==', '/=', '=:=', '=/=', '<', '=<', '>', '>='],
     binop(Ops, Vs).
@@ -523,7 +541,7 @@ range_tests(_Config) ->
     inside = range_big(MinSmall),
     inside = range_big(-1 bsl 58),
     inside = range_big(0),
-    inside = range_barely_small(17.75),
+    inside = range_big(17.75),
     inside = range_big(1 bsl 58),
     inside = range_big(MaxSmall),
 
@@ -531,6 +549,39 @@ range_tests(_Config) ->
     greater = range_big(1 bsl 64),
     greater = range_big(float(1 bsl 64)),
 
+    inside = int_range_1(id(-100_000)),
+    inside = int_range_1(id(-10)),
+    inside = int_range_1(id(100)),
+    inside = int_range_1(id(100_000)),
+
+    outside = int_range_1(id(atom)),
+    outside = int_range_1(id(-1 bsl 60)),
+    outside = int_range_1(id(-100_001)),
+    outside = int_range_1(id(100_001)),
+    outside = int_range_1(id(1 bsl 60)),
+
+    inside = int_range_2(id(1)),
+    inside = int_range_2(id(42)),
+    inside = int_range_2(id(16#f000_0000)),
+
+    outside = int_range_2(id([a,list])),
+    outside = int_range_2(id(0)),
+    outside = int_range_1(id(-1 bsl 60)),
+    outside = int_range_1(id(1 bsl 60)),
+
+    inside = int_range_3(id(1 bsl 28)),
+    inside = int_range_3(id((1 bsl 28) + 1)),
+    inside = int_range_3(id((1 bsl 33) + 555)),
+    inside = int_range_3(id((1 bsl 58) - 1)),
+    inside = int_range_3(id(1 bsl 58)),
+
+    outside = int_range_3(id({a,tuple})),
+    outside = int_range_3(id(-1 bsl 60)),
+    outside = int_range_3(id(-1000)),
+    outside = int_range_3(id(100)),
+    outside = int_range_3(id((1 bsl 58) + 1)),
+    outside = int_range_3(id(1 bsl 60)),
+
     ok.
 
 range(X) ->
@@ -683,9 +734,261 @@ range_big_2(X) when (-1 bsl 59) - 1 =< X, X =< 1 bsl 59 ->
 range_big_2(_) ->
     outside.
 
+int_range_1(X) when is_integer(X), -100_000 =< X, X =< 100_000 ->
+    inside;
+int_range_1(_) ->
+    outside.
+
+int_range_2(X) when is_integer(X), 1 =< X, X =< 16#f000_0000 ->
+    inside;
+int_range_2(_) ->
+    outside.
+
+int_range_3(X) when is_integer(X), 1 bsl 28 =< X, X =< 1 bsl 58 ->
+    inside;
+int_range_3(_) ->
+    outside.
+
+combined_relops(_Config) ->
+    other = test_tok_char(-1 bsl 64),
+    other = test_tok_char($A - 1),
+
+    var = test_tok_char($A),
+    var = test_tok_char($B),
+    var = test_tok_char($P),
+    var = test_tok_char($Y),
+    var = test_tok_char($Z),
+
+    other = test_tok_char($Z + 1),
+
+    var = tok_char($_),
+    other = tok_char(float($_)),
+
+    other = test_tok_char(1 bsl 64),
+
+    other = test_tok_char(atom),
+    other = test_tok_char(self()),
+
+    %%
+    b = ge_ge_int_range_1(-200),
+    b = ge_ge_int_range_1(-101),
+
+    a = ge_ge_int_range_1(-100),
+    a = ge_ge_int_range_1(-50),
+    a = ge_ge_int_range_1(-10),
+
+    b = ge_ge_int_range_1(-9),
+    b = ge_ge_int_range_1(-6),
+
+    a = ge_ge_int_range_1(-5),
+
+    b = ge_ge_int_range_1(-4),
+    b = ge_ge_int_range_1(0),
+    b = ge_ge_int_range_1(42),
+
+    %%
+    b = ge_ge_int_range_2(-1 bsl 59),
+
+    a = ge_ge_int_range_2((-1 bsl 59) + 1),
+    a = ge_ge_int_range_2(-1 bsl 58),
+    a = ge_ge_int_range_2(-1000),
+    a = ge_ge_int_range_2(1 bsl 58),
+    a = ge_ge_int_range_2((1 bsl 59) - 10),
+
+    b = ge_ge_int_range_2((1 bsl 59) - 9),
+
+    a = ge_ge_int_range_2((1 bsl 59) - 5),
+
+    b = ge_ge_int_range_2((1 bsl 59) - 4),
+    b = ge_ge_int_range_2((1 bsl 59) - 1),
+
+    %%
+    b = ge_ge_int_range_3(-1 bsl 59),
+
+    b = ge_ge_int_range_3((-1 bsl 59) + 1),
+    b = ge_ge_int_range_3(-1 bsl 58),
+    b = ge_ge_int_range_3(-1000),
+    b = ge_ge_int_range_3(1 bsl 58),
+
+    a = ge_ge_int_range_3((1 bsl 59) - 20),
+    a = ge_ge_int_range_3((1 bsl 59) - 15),
+    a = ge_ge_int_range_3((1 bsl 59) - 10),
+
+    b = ge_ge_int_range_3((1 bsl 59) - 9),
+
+    a = ge_ge_int_range_3((1 bsl 59) - 5),
+
+    b = ge_ge_int_range_3((1 bsl 59) - 4),
+    b = ge_ge_int_range_3((1 bsl 59) - 1),
+
+    %%
+    b = ge_ge_int_range_4(-1 bsl 59),
+
+    a = ge_ge_int_range_4((-1 bsl 59) + 1),
+    a = ge_ge_int_range_4((-1 bsl 59) + 3),
+    a = ge_ge_int_range_4((-1 bsl 59) + 5),
+
+    b = ge_ge_int_range_4((-1 bsl 59) + 6),
+    b = ge_ge_int_range_4((-1 bsl 59) + 9),
+
+    a = ge_ge_int_range_4((-1 bsl 59) + 10),
+
+    b = ge_ge_int_range_4((-1 bsl 59) + 11),
+
+    b = ge_ge_int_range_4(0),
+    b = ge_ge_int_range_4(1000),
+
+    b = ge_ge_int_range_4((1 bsl 59) - 1),
+
+    %% Test a sequence that can't occur in optimized code:
+    %%   is_ge Fail Src 10
+    %%   is_ge Fail Src 5
+    Module = {?FUNCTION_NAME,[{test,1}],[],
+              [{function, test, 1, 2,
+                [{label,1},
+                 {line,[{location,"t.erl",4}]},
+                 {func_info,{atom,?FUNCTION_NAME},{atom,test},1},
+                 {label,2},
+                 {test,is_ge,{f,4},
+                  [{tr,{x,0},{t_integer,{0,1000}}},
+                   {integer,10}]},
+                 {test,is_ge,
+                  {f,3},
+                  [{tr,{x,0},{t_integer,{0,1000}}},
+                   {integer,5}]},
+                 {label,3},
+                 {move,{atom,a},{x,0}},
+                 return,
+                 {label,4},
+                 {move,{atom,b},{x,0}},
+                 return]}],
+              5},
+
+    {ok,Mod,Code} = compile:forms(Module, [from_asm,time,report]),
+    {module,Mod} = code:load_binary(Mod, Mod, Code),
+
+    b = Mod:test(0),
+    b = Mod:test(5),
+    b = Mod:test(9),
+
+    a = Mod:test(10),
+    a = Mod:test(11),
+    a = Mod:test(1000),
+
+    true = code:delete(Mod),
+    _ = code:purge(Mod),
+
+    LargeMap = #{K => K || K <- lists:seq(1, 64)},
+    Ref = make_ref(),
+    [{}] = empty_map_or_nil(id({})),
+    [{a,b}] = empty_map_or_nil(id({a,b})),
+    [#{a := b}] = empty_map_or_nil(id(#{a => b})),
+    [LargeMap] = empty_map_or_nil(id(LargeMap)),
+    [Ref] = empty_map_or_nil(id(Ref)),
+    [[a,b,c]] = empty_map_or_nil(id([a,b,c])),
+    [a] = empty_map_or_nil(id(a)),
+    [42] = empty_map_or_nil(id(42)),
+    [] = empty_map_or_nil(id(#{})),
+    [] = empty_map_or_nil(id([])),
+
+    ok.
+
+test_tok_char(C) ->
+    Result = tok_char(C),
+    if
+        is_integer(C) ->
+            Result = tok_char(float(C)),
+            Result = tok_char_int(C),
+            if
+                C band 16#FFFF =:= C ->
+                    Result = tok_char_int_range(C);
+                true ->
+                    Result
+            end;
+        true ->
+            Result
+    end.
+
+%% is_ge + is_lt
+tok_char(C) when $A =< C, C =< $Z ->
+    var;
+tok_char($_) ->
+    var;
+tok_char(_) ->
+    other.
+
+%% is_ge + is_ge
+tok_char_int(C) when $A =< C, C =< $Z ->
+    var;
+tok_char_int($_) ->
+    var;
+tok_char_int(_) ->
+    other.
+
+%% is_ge + is_ge
+tok_char_int_range(C) when $A =< C, C =< $Z ->
+    var;
+tok_char_int_range($_) ->
+    var;
+tok_char_int_range(_) ->
+    other.
+
+%% is_ge + is_ge
+ge_ge_int_range_1(X) when -100 =< X, X =< -10 ->
+    a;
+ge_ge_int_range_1(-5) ->
+    a;
+ge_ge_int_range_1(_) ->
+    b.
+
+ge_ge_int_range_2(X) when (-1 bsl 59) + 1 =< X, X =< (1 bsl 59) - 10 ->
+    a;
+ge_ge_int_range_2((1 bsl 59) - 5) ->
+    a;
+ge_ge_int_range_2(_) ->
+    b.
+
+ge_ge_int_range_3(X) when (1 bsl 59) - 20 =< X, X =< (1 bsl 59) - 10 ->
+    a;
+ge_ge_int_range_3((1 bsl 59) - 5) ->
+    a;
+ge_ge_int_range_3(_) ->
+    b.
+
+ge_ge_int_range_4(X) when (-1 bsl 59) + 1 =< X, X =< (-1 bsl 59) + 5 ->
+    a;
+ge_ge_int_range_4((-1 bsl 59) + 10) ->
+    a;
+ge_ge_int_range_4(_) ->
+    b.
+
+%% GH-8325. An inverted test with an empty map would fail for
+%% non-boxed terms.
+empty_map_or_nil(V) when V =:= #{}; V =:= [] ->
+    [];
+empty_map_or_nil(V) ->
+    [V].
+
 %% Tests operators where type hints are significant.
 typed_relop(Config) when is_list(Config) ->
     _ = [compare_integer_pid(1 bsl N) || N <- lists:seq(1, 64)],
+
+    {error,<<7:3>>} = compare_bitstring({text, <<7:3>>, 0}),
+    {error,<<0:8>>} = compare_bitstring({text, <<0:8>>, 0}),
+    {error,<<0:9>>} = compare_bitstring({text, <<0:9>>, 0}),
+    {text, 42} = compare_bitstring({text, <<0:3>>, 42}),
+
+    {error,<<7:3>>} = compare_bitstring({binary, <<7:3>>, 0}),
+    {error,<<0:8>>} = compare_bitstring({binary, <<0:8>>, 0}),
+    {error,<<0:9>>} = compare_bitstring({binary, <<0:9>>, 0}),
+    {binary, 42} = compare_bitstring({binary, <<0:3>>, 42}),
+
+    negative = classify_value(id(-1 bsl 128)),
+    other = classify_value(id(0)),
+    other = classify_value(id(42)),
+    other = classify_value(id(1 bsl 64)),
+    other = classify_value(id(a)),
+
     ok.
 
 compare_integer_pid(N) when is_integer(N) ->
@@ -696,6 +999,109 @@ compare_integer_pid(N) when is_integer(N) ->
         N < Immed -> ok
     end.
 
+%% GH-7433. Equality and non-equality tests with a bitstring could fail when it
+%% should succeed and vice versa.
+compare_bitstring({text, Res, _Data}) when is_bitstring(Res), Res =/= <<0:3>> ->
+    {error, Res};
+compare_bitstring({binary, Res, _Data}) when is_bitstring(Res), Res =/= <<0:3>> ->
+    {error, Res};
+compare_bitstring({binary, _Res, Data}) ->
+    {binary, Data};
+compare_bitstring({text, _Res, Data}) ->
+    {text, Data}.
+
+classify_value(N) when is_integer(N), N < 0 ->
+    negative;
+classify_value(_N) ->
+    other.
+
+term_equivalence(_Config) ->
+    %% Term equivalence has been tested before in this suite, but we need to
+    %% massage some edge cases that cannot easily be put into the other test
+    %% cases.
+    <<PosZero/float>> = id(<<0:1,0:63>>),
+    <<NegZero/float>> = id(<<1:1,0:63>>),
+
+    +0.0 = id(PosZero),
+    -0.0 = id(NegZero),
+
+    -1 = erts_internal:cmp_term(NegZero, PosZero),
+    1 = erts_internal:cmp_term(PosZero, NegZero),
+
+    -1 = cmp_float(NegZero, PosZero),
+    1 = cmp_float(PosZero, NegZero),
+
+    Floats = [0.0, -0.0, 4711.0, -4711.0, 12.0, 943.0],
+
+    [true = (cmp_float(A, B) =:= erts_internal:cmp_term(A, B)) ||
+     A <- Floats, B <- Floats],
+
+    ok.
+
+cmp_float(A0, B0) ->
+    A = float_comparable(A0),
+    B = float_comparable(B0),
+    if
+        A < B -> -1;
+        A > B -> +1;
+        A =:= B -> 0
+    end.
+
+%% Converts a float to a number which, when compared with other such converted
+%% floats, is ordered the same as '<' on the original inputs aside from the
+%% fact that -0.0 < +0.0 as required by the term equivalence order.
+%%
+%% This has been proven correct with the SMT-LIB model below:
+%%
+%% (define-const SignBit_bv (_ BitVec 64) #x8000000000000000)
+%% 
+%% ; Two finite floats X and Y of unknown value
+%% (declare-const X (_ FloatingPoint 11 53))
+%% (declare-const Y (_ FloatingPoint 11 53))
+%% (assert (= false (fp.isInfinite X) (fp.isInfinite Y)
+%%            (fp.isNaN X) (fp.isNaN Y)))
+%% 
+%% ; ... the bit representations of the aforementioned floats. The Z3 floating-
+%% ; point extension lacks a way to directly bit-cast a vector to a float, so
+%% ; we rely on equivalence here.
+%% (declare-const X_bv (_ BitVec 64))
+%% (declare-const Y_bv (_ BitVec 64))
+%% (assert (= ((_ to_fp 11 53) X_bv) X))
+%% (assert (= ((_ to_fp 11 53) Y_bv) Y))
+%% 
+%% ; The bit hack we're going to test
+%% (define-fun float_sortable ((value (_ BitVec 64))) (_ BitVec 64)
+%% (ite (distinct (bvand value SignBit_bv) SignBit_bv)
+%%  (bvxor value SignBit_bv)
+%%  (bvnot value)))
+%% 
+%% (define-fun float_bv_lt ((LHS (_ BitVec 64))
+%%                          (RHS (_ BitVec 64))) Bool
+%%  (bvult (float_sortable LHS) (float_sortable RHS)))
+%% 
+%% (push 1)
+%%   ; When either of X or Y are non-zero, (X < Y) = (bvX < bvY)
+%%   (assert (not (and (fp.isZero X) (fp.isZero Y))))
+%%   (assert (distinct (fp.lt X Y) (float_bv_lt X_bv Y_bv)))
+%%   (check-sat) ; unsat, proving by negation that the above always holds
+%% (pop 1)
+%% 
+%% (push 1)
+%%   ; Negative zero should sort lower than positive zero
+%%   (assert (and (fp.isNegative X) (fp.isPositive Y)
+%%                (fp.isZero X) (fp.isZero Y)))
+%%   (assert (not (float_bv_lt X_bv Y_bv)))
+%%   (check-sat) ; unsat
+%% (pop 1)
+float_comparable(V0) ->
+    Sign = 16#8000000000000000,
+    Mask = 16#FFFFFFFFFFFFFFFF,
+    <<V_bv:64/unsigned>> = <<V0/float>>,
+    case V_bv band Sign of
+        0 -> (V_bv bxor Sign) band Mask;
+        Sign -> (bnot V_bv) band Mask
+    end.
+
 %%%
 %%% Utilities.
 %%%
@@ -712,7 +1118,10 @@ value({atom,_,X})    -> X;
 value({tuple,_,Es}) ->
     list_to_tuple(lists:map(fun(X) -> value(X) end, Es));
 value({cons,_,H,T}) ->
-    [value(H) | value(T)].
+    [value(H) | value(T)];
+value(Other) ->
+    {value,Value,_} = erl_eval:expr(Other, erl_eval:new_bindings()),
+    Value.
 
 repeat(_, 0) -> ok;
 repeat(Fun, N) ->
