@@ -1,7 +1,7 @@
 /*
  * %CopyrightBegin%
  *
- * Copyright Ericsson AB 1998-2021. All Rights Reserved.
+ * Copyright Ericsson AB 1998-2024. All Rights Reserved.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -771,6 +771,11 @@ static void main_loop(void)
     }
 #endif
 
+    /* Satisfy CodeChecker.  Already done in init_workers() */
+    *wsizes[0] = 0;
+    *wsizes[1] = 0;
+    *wsizes[2] = 0;
+
     for(;;) {
 #ifdef WIN32
 	num_handles = 0;
@@ -1041,14 +1046,23 @@ static void main_loop(void)
 
 static void init_workers(int max)
 {
+    size_t alloc_size = sizeof(Worker) * max;
+
     max_workers = max;
     num_busy_workers = 0;
     num_free_workers = 0;
     num_stalled_workers = 0;
 
-    busy_workers = ALLOC(sizeof(Worker) * max_workers);
-    free_workers = ALLOC(sizeof(Worker) * max_workers);
-    stalled_workers = ALLOC(sizeof(Worker) * max_workers);
+    busy_workers = ALLOC(alloc_size);
+    free_workers = ALLOC(alloc_size);
+    stalled_workers = ALLOC(alloc_size);
+
+#ifdef DEBUG
+    memset(busy_workers, 0, alloc_size);
+    memset(free_workers, 0, alloc_size);
+    memset(stalled_workers, 0, alloc_size);;
+#endif
+
 #ifndef WIN32
     init_signals();
 #endif
@@ -1754,12 +1768,11 @@ static int worker_loop(void)
 	/* Decode the request... */
 	serial = get_serial(req);
 	if (OP_CONTROL == (op = get_op(req))) {
-	    CtlType ctl;
 	    if (serial != INVALID_SERIAL) {
 		DEBUGF(1, ("Worker got invalid serial: %d.", serial));
 		exit(0);
 	    }
-	    switch (ctl = get_ctl(req)) {
+	    switch (get_ctl(req)) {
 	    case SETOPT_DEBUG_LEVEL:
 		debug_level = get_debug_level(req);
 		DEBUGF(debug_level, 
