@@ -1,7 +1,7 @@
 %%
 %% %CopyrightBegin%
 %%
-%% Copyright Ericsson AB 2001-2024. All Rights Reserved.
+%% Copyright Ericsson AB 2001-2025. All Rights Reserved.
 %%
 %% Licensed under the Apache License, Version 2.0 (the "License");
 %% you may not use this file except in compliance with the License.
@@ -323,7 +323,12 @@ testCompactBitString(Config, Rule, Opts) ->
     testCompactBitString:otp_4869(Rule).
 
 testPrimStrings(Config) ->
-    test(Config, fun testPrimStrings/3, [ber,{ber,[der]},per,uper]).
+    test(Config, fun testPrimStrings/3, [ber,{ber,[der]},per,uper,jer]).
+testPrimStrings(Config, jer=Rule, Opts) ->
+    Files = ["PrimStrings", "BitStr"],
+    asn1_test_lib:compile_all(Files, Config, [Rule|Opts]),
+    testPrimStrings_cases(Rule, Opts),
+    testPrimStrings:more_strings(Rule);
 testPrimStrings(Config, Rule, Opts) ->
     LegacyOpts = [legacy_erlang_types|Opts],
     Files = ["PrimStrings", "BitStr"],
@@ -342,11 +347,16 @@ testPrimStrings_cases(Rule, Opts) ->
     testPrimStrings:octet_string(Rule),
     testPrimStrings:numeric_string(Rule),
     testPrimStrings:other_strings(Rule),
-    testPrimStrings:universal_string(Rule),
-    testPrimStrings:bmp_string(Rule),
-    testPrimStrings:times(Rule),
-    testPrimStrings:utf8_string(Rule),
-    testPrimStrings:fragmented(Rule).
+    case Rule of
+        jer ->
+            ok;
+        _ ->
+            testPrimStrings:universal_string(Rule),
+            testPrimStrings:bmp_string(Rule),
+            testPrimStrings:times(Rule),
+            testPrimStrings:utf8_string(Rule),
+            testPrimStrings:fragmented(Rule)
+    end.
 
 testExternal(Config) -> test(Config, fun testExternal/3).
 testExternal(Config, Rule, Opts) ->
