@@ -1,7 +1,7 @@
 %%
 %% %CopyrightBegin%
 %%
-%% Copyright Ericsson AB 2001-2018. All Rights Reserved.
+%% Copyright Ericsson AB 2001-2024. All Rights Reserved.
 %%
 %% Licensed under the Apache License, Version 2.0 (the "License");
 %% you may not use this file except in compliance with the License.
@@ -20,6 +20,7 @@
 %% Purpose: Information about the Erlang built-in functions.
 
 -module(erl_bifs).
+-moduledoc false.
 
 -export([is_pure/3, is_safe/3, is_exit_bif/3]).
 
@@ -80,10 +81,12 @@ is_pure(erlang, 'or', 2) -> true;
 is_pure(erlang, 'rem', 2) -> true;
 is_pure(erlang, 'xor', 2) -> true;
 is_pure(erlang, abs, 1) -> true;
+is_pure(erlang, atom_to_binary, 1) -> true;
 is_pure(erlang, atom_to_binary, 2) -> true;
 is_pure(erlang, atom_to_list, 1) -> true;
 is_pure(erlang, binary_part, 2) -> true;
 is_pure(erlang, binary_part, 3) -> true;
+is_pure(erlang, binary_to_atom, 1) -> true;
 is_pure(erlang, binary_to_atom, 2) -> true;
 is_pure(erlang, binary_to_float, 1) -> true;
 is_pure(erlang, binary_to_integer, 1) -> true;
@@ -137,13 +140,15 @@ is_pure(erlang, round, 1) -> true;
 is_pure(erlang, setelement, 3) -> true;
 is_pure(erlang, size, 1) -> true;
 is_pure(erlang, split_binary, 2) -> true;
-is_pure(erlang, term_to_binary, 1) -> true;
 is_pure(erlang, tl, 1) -> true;
 is_pure(erlang, trunc, 1) -> true;
 is_pure(erlang, tuple_size, 1) -> true;
 is_pure(erlang, tuple_to_list, 1) -> true;
 is_pure(lists, append, 2) -> true;
 is_pure(lists, subtract, 2) -> true;
+is_pure(maps, get, 2) -> true;
+is_pure(maps, is_key, 2) -> true;
+is_pure(maps, new, 0) -> true;
 is_pure(math, acos, 1) -> true;
 is_pure(math, acosh, 1) -> true;
 is_pure(math, asin, 1) -> true;
@@ -183,8 +188,13 @@ is_pure(_, _, _) -> false.
 %%	and does not affect the state (although the value it returns
 %%	might depend on the state).
 %%
-%%      Note: is_function/2 and is_record/3 are NOT safe: is_function(X, foo)
-%%      and is_record(X, foo, bar) will fail.
+%%      NOTES
+%%
+%%      is_function/2 is not safe: is_function(X, foo) will fail.
+%%
+%%      is_record/3 is not safe: is_record(X, foo, bar) will fail.
+%%
+%%      erlang:make_fun/3 is safe: erlang:make_fun3(foo, bar, baz) will fail.
 
 -spec is_safe(atom(), atom(), arity()) -> boolean().
 
@@ -218,7 +228,6 @@ is_safe(erlang, is_port, 1) -> true;
 is_safe(erlang, is_reference, 1) -> true;
 is_safe(erlang, is_tuple, 1) -> true;
 is_safe(erlang, make_ref, 0) -> true;
-is_safe(erlang, make_fun, 3) -> true;
 is_safe(erlang, max, 2) -> true;
 is_safe(erlang, min, 2) -> true;
 is_safe(erlang, node, 0) -> true;
