@@ -1,7 +1,7 @@
 %%
 %% %CopyrightBegin%
 %%
-%% Copyright Ericsson AB 2010-2024. All Rights Reserved.
+%% Copyright Ericsson AB 2010-2025. All Rights Reserved.
 %%
 %% Licensed under the Apache License, Version 2.0 (the "License");
 %% you may not use this file except in compliance with the License.
@@ -447,7 +447,7 @@ run(List)
 
 run(List, Dir)
   when is_list(List) ->
-    Path = filename:join([code:lib_dir(diameter, src), "dict", ?base]),
+    Path = filename:join([?LIB_DIR(diameter, src), "dict", ?base]),
     {ok, Bin} = file:read_file(Path),
     ?RUN([{{?MODULE, F, [{Bin, Dir}]}, 180000} || F <- List]);
 
