@@ -1,7 +1,7 @@
 %%
 %% %CopyrightBegin%
 %%
-%% Copyright Ericsson AB 1999-2016. All Rights Reserved.
+%% Copyright Ericsson AB 1999-2024. All Rights Reserved.
 %%
 %% Licensed under the Apache License, Version 2.0 (the "License");
 %% you may not use this file except in compliance with the License.
@@ -21,6 +21,40 @@
 %
 
 -module(odbc).
+-moduledoc """
+Erlang ODBC application
+
+This application provides an Erlang interface to communicate with relational
+SQL-databases. It is built on top of Microsofts ODBC interface and therefore
+requires that you have an ODBC driver to the database that you want to connect
+to.
+
+> #### Note {: .info }
+>
+> The functions `first/[1,2]`, `last/[1,2]`, `next/[1,2]`, `prev[1,2]` and
+> `select/[3,4]` assumes there is a result set associated with the connection to
+> work on. Calling the function `select_count/[2,3]` associates such a result
+> set with the connection. Calling select_count again will remove the current
+> result set association and create a new one. Calling a function which dose not
+> operate on an associated result sets, such as `sql_query/[2,3]`, will remove
+> the current result set association.
+>
+> Alas some drivers only support sequential traversal of the result set, e.i.
+> they do not support what in the ODBC world is known as scrollable cursors.
+> This will have the effect that functions such as `first/[1,2]`, `last/[1,2]`,
+> `prev[1,2]`, etc will return `{error, driver_does_not_support_function}`
+
+## Error Handling
+
+The error handling strategy and possible errors sources are described in the
+Erlang ODBC [User's Guide.](error_handling.md)
+
+## References
+
+\[1]: Microsoft ODBC 3.0, Programmer's Reference and SDK Guide  
+See also http://msdn.microsoft.com/
+""".
+-moduledoc(#{titles => [{type,<<"Types used in ODBC application">>}]}).
 
 -behaviour(gen_server).
 
@@ -45,6 +79,72 @@
 -export([init/1, handle_call/3, handle_cast/2, handle_info/2, 
 	 terminate/2, code_change/3]).
 
+-doc "Opaque reference to an ODBC connection as returnded by connect/2.".
+-doc(#{title => <<"Types used in ODBC application">>}).
+-opaque connection_reference() :: pid().
+-doc "Name of column in the result set.".
+-doc(#{title => <<"Types used in ODBC application">>}).
+-type col_name()             :: string().
+-doc """
+A tuple, with the number of elements selected form columns in a database row,
+containg the values of the columns such as `{value(), value() ... value()} `.
+""".
+-doc(#{title => <<"Types used in ODBC application">>}).
+-type row()                  :: tuple().
+-doc "Erlang data type that corresponds to the ODBC data type being handled.".
+-doc(#{title => <<"Types used in ODBC application">>}).
+-type value()                :: null | term().
+-doc "Return value for queries that select data from database tabels.".
+-doc(#{title => <<"Types used in ODBC application">>}).
+-type selected()             :: {selected, [col_name()], [row()]}.
+-doc "Return value for queries that update database tables.".
+-doc(#{title => <<"Types used in ODBC application">>}).
+-type updated()              :: {updated, n_rows()}.
+-doc """
+The number of affected rows for UPDATE, INSERT, or DELETE queries. For other
+query types the value is driver defined, and hence should be ignored.
+""".
+-doc(#{title => <<"Types used in ODBC application">>}).
+-type n_rows()               :: integer().
+-doc """
+Data type used by ODBC, to learn which Erlang data type corresponds to an ODBC
+data type see the Erlang to ODBC data type [mapping](databases.md#type) in the
+User's Guide.
+""".
+-doc(#{title => <<"Types used in ODBC application">>}).
+-type odbc_data_type()       ::  sql_integer | sql_smallint | sql_tinyint |
+                                 {sql_decimal, Precision::integer(), Scale::integer()} |
+                                 {sql_numeric, Precision::integer(), Scale::integer()} |
+                                 {sql_char, Size::integer()} |
+                                 {sql_wchar, Size::integer()} |
+                                 {sql_varchar, Size::integer()} |
+                                 {sql_wvarchar, Size::integer()}|
+                                 {sql_float, Precision::integer()} |
+                                 {sql_wlongvarchar, Size::integer()} |
+                                 {sql_float, Precision::integer()} |
+                                 sql_real | sql_double | sql_bit | atom().
+-doc """
+An explanation of what went wrong. For common errors there will be atom
+decriptions.
+""".
+-doc(#{title => <<"Types used in ODBC application">>}).
+-type common_reason()        :: connection_closed | extended_error() | term().
+-doc """
+extended error type with ODBC and native database error codes, as well as the
+base reason that would have been returned had extended_errors not been enabled.
+""".
+-doc(#{title => <<"Types used in ODBC application">>}).
+-type extended_error()       :: {string(), integer(), term()}.
+
+-export_type([connection_reference/0,
+              col_name/0, row/0,
+              selected/0,
+              updated/0,
+              n_rows/0,
+              odbc_data_type/0,
+              common_reason/0,
+              extended_error/0]).
+
 %%--------------------------------------------------------------------------
 %% Internal state
 -record(state, {erlang_port,                 % The port to the c-program
@@ -52,7 +152,7 @@
 		owner,                       % Pid of the connection owner
 		result_set = undefined,      % exists | undefined
 		auto_commit_mode = on,       % on | off
-		%% Indicates if first, last and "select absolut"
+		%% Indicates if first, last and "select absolute"
 		%% is supported by the odbc driver.
 		absolute_pos,                % true | false  
 		%% Indicates if prev and "select relative"
@@ -75,23 +175,32 @@
 %%%  API
 %%%=========================================================================
 
+%%--------------------------------------------------------------------
+-doc(#{equiv => start/1}).
+-spec start() -> ok | {error, Reason} when
+      Reason ::term().
+%%--------------------------------------------------------------------
+start() ->
+    application:start(odbc).
 
 %%--------------------------------------------------------------------
-%% Function: start([, Type]) -> ok
-%%
-%%  Type =  permanent | transient | temporary
+-doc """
+Starts the odbc application. Default type is temporary.
+[See application(3)](`m:application`)
+""".
+-spec start(Type) -> ok | {error, Reason} when
+      Type :: permanent | transient | temporary,
+      Reason ::term().
 %%
 %% Description: Starts the inets application. Default type
 %% is temporary. see application(3)
 %%--------------------------------------------------------------------
-start() -> 
-    application:start(odbc).
-
 start(Type) -> 
     application:start(odbc, Type).
 
 %%--------------------------------------------------------------------
-%% Function: stop() -> ok
+-doc "Stops the odbc application. [See application(3)](`m:application`)".
+-spec stop() -> ok.
 %%
 %% Description: Stops the odbc application.
 %%--------------------------------------------------------------------
@@ -99,8 +208,85 @@ stop() ->
     application:stop(odbc).
 
 %%-------------------------------------------------------------------------
-%% connect(ConnectionStr, Options) -> {ok, ConnectionReferense} |
-%%                                    {error, Reason}
+-doc """
+Opens a connection to the database. The connection is associated with the
+process that created it and can only be accessed through it. This function may
+spawn new processes to handle the connection. These processes will terminate if
+the process that created the connection dies or if you call disconnect/1.
+
+If automatic commit mode is turned on, each query will be considered as an
+individual transaction and will be automatically committed after it has been
+executed. If you want more than one query to be part of the same transaction the
+automatic commit mode should be turned off. Then you will have to call commit/3
+explicitly to end a transaction.
+
+The default timeout is infinity
+
+If the option binary_strings is turned on all strings will be returned as
+binaries and strings inputted to param_query will be expected to be binaries.
+The user needs to ensure that the binary is in an encoding that the database
+expects. By default this option is turned off.
+
+As default result sets are returned as a lists of tuples. The `TupleMode` option
+still exists to keep some degree of backwards compatibility. If the option is
+set to off, result sets will be returned as a lists of lists instead of a lists
+of tuples.
+
+Scrollable cursors are nice but causes some overhead. For some connections speed
+might be more important than flexible data access and then you can disable
+scrollable cursor for a connection, limiting the API but gaining speed.
+
+> #### Note {: .info }
+>
+> Turning the scrollable_cursors option off is noted to make old odbc-drivers
+> able to connect that will otherwise fail.
+
+If trace mode is turned on this tells the ODBC driver to write a trace log to
+the file SQL.LOG that is placed in the current directory of the erlang emulator.
+This information may be useful if you suspect there might be a bug in the erlang
+ODBC application, and it might be relevant for you to send this file to our
+support. Otherwise you will probably not have much use of this.
+
+> #### Note {: .info }
+>
+> For more information about the `ConnectStr` see description of the function
+> SQLDriverConnect in \[1].
+
+The `extended_errors` option enables extended ODBC error information when an
+operation fails. Rather than returning `{error, Reason}`, the failing function
+will return `{error, {ODBCErrorCode, NativeErrorCode, Reason}}`. Note that this
+information is probably of little use when writing database-independent code,
+but can be of assistance in providing more sophisticated error handling when
+dealing with a known underlying database.
+
+- `ODBCErrorCode` is the ODBC error string returned by the ODBC driver.
+- `NativeErrorCode` is the numeric error code returned by the underlying
+  database. The possible values and their meanings are dependent on the database
+  being used.
+- `Reason` is as per the `Reason` field when extended errors are not enabled.
+
+> #### Note {: .info }
+>
+> The current implementation spawns a port program written in C that utilizes
+> the actual ODBC driver. There is a default timeout of 5000 msec for this port
+> program to connect to the Erlang ODBC application. This timeout can be changed
+> by setting an application specific environment variable 'port_timeout' with
+> the number of milliseconds for the ODBC application. E.g.: \[\{odbc,
+> [\{port_timeout, 60000\}]\}] to set it to 60 seconds.
+""".
+-spec connect(ConnectionStr, Options) -> {ok, ConnectionReferense} |
+          {error, Reason} when
+      ConnectionStr :: string(),
+      Options :: [{auto_commit, on | off} |
+                  {timeout, erlang:timeout()} |
+                  {binary_strings, on | off} |
+                  {tuple_row, on | off} |
+                  {scrollable_cursors, on | off} |
+                  {trace_driver, on | off} |
+                  {extended_errors, on | off}],
+      ConnectionReferense :: connection_reference(),
+      Reason :: port_program_executable_not_found | common_reason().
+
 %% Description: Spawns an erlang control process that will open a port
 %%              to a c-process that uses the ODBC API to open a connection
 %%              to the database. 
@@ -119,7 +305,16 @@ connect(ConnectionStr, Options) when is_list(ConnectionStr), is_list(Options) ->
     end.
 
 %%--------------------------------------------------------------------------
-%% disconnect(ConnectionReferense) -> ok | {error, Reason}
+-doc """
+Closes a connection to a database. This will also terminate all processes that
+may have been spawned when the connection was opened. This call will always
+succeed. If the connection cannot be disconnected gracefully it will be brutally
+killed. However you may receive an error message as result if you try to
+disconnect a connection started by another process.[](){: #describe_table }
+""".
+-spec disconnect(ConnectionReferense) -> ok | {error, Reason} when
+      ConnectionReferense :: connection_reference(),
+      Reason :: process_not_owner_of_odbc_connection | extended_error().
 %%                                    
 %% Description: Disconnects from the database and terminates both the erlang
 %%              control process and the database handling c-process. 
@@ -129,11 +324,11 @@ disconnect(ConnectionReference) when is_pid(ConnectionReference)->
     case call(ConnectionReference, {disconnect, ODBCCmd}, 5000) of 
 	{error, connection_closed} ->
 	    %% If the connection has already been closed the effect of
-	    %% disconnect has already been acomplished
+	    %% disconnect has already been accomplished
 	    ok; 
 	%% Note a time out of this call will return ok, as disconnect
 	%% will always succeed, the time out is to make sure
-	%% the connection is killed brutaly if it will not be shut down
+	%% the connection is killed brutally if it will not be shut down
 	%% gracefully.
 	ok ->
 	    ok;
@@ -144,14 +339,28 @@ disconnect(ConnectionReference) when is_pid(ConnectionReference)->
     end. 
 	    
 %%--------------------------------------------------------------------------
-%% commit(ConnectionReference, CommitMode, <TimeOut>) -> ok | {error,Reason}
-%%                                    
-%% Description: Commits or rollbacks a transaction. Needed on connections
-%%              where automatic commit is turned off.  
+-doc(#{equiv => commit/3}).
+-spec commit(ConnectionReference, CommitMode) -> ok | {error, Reason} when
+      ConnectionReference :: connection_reference(),
+      CommitMode ::  commit | rollback,
+      Reason :: not_an_explicit_commit_connection |
+                process_not_owner_of_odbc_connection | common_reason().
 %%--------------------------------------------------------------------------
 commit(ConnectionReference, CommitMode) ->
     commit(ConnectionReference, CommitMode, ?DEFAULT_TIMEOUT).
 
+%%--------------------------------------------------------------------------
+-doc """
+Commits or rollbacks a transaction. Needed on connections where automatic commit
+is turned off.
+""".
+-spec commit(ConnectionReference, CommitMode, TimeOut) -> ok | {error, Reason} when
+      ConnectionReference :: connection_reference(),
+      CommitMode ::  commit | rollback,
+      TimeOut :: erlang:timeout(),
+      Reason :: not_an_explicit_commit_connection |
+                process_not_owner_of_odbc_connection | common_reason().
+%%--------------------------------------------------------------------------
 commit(ConnectionReference, commit, infinity) 
   when is_pid(ConnectionReference) ->
     ODBCCmd = [?COMMIT_TRANSACTION, ?COMMIT],
@@ -173,16 +382,41 @@ commit(ConnectionReference, rollback, TimeOut)
     call(ConnectionReference, {commit, ODBCCmd}, TimeOut).
 
 %%--------------------------------------------------------------------------
-%% sql_query(ConnectionReference, SQLQuery, <TimeOut>) -> {updated, NRows} |
-%%			       {selected, ColNames, Rows} | {error, Reason} 
+-doc(#{equiv => sql_query/3}).
+-spec sql_query(ConnectionReference, SQLQuery) -> Result | {error, Reason}  when
+      ConnectionReference :: connection_reference(),
+      SQLQuery :: string(),
+      Result  :: updated() | selected(),
+      Reason  :: process_not_owner_of_odbc_connection | common_reason().
 %%                                    
-%% Description: Executes a SQL query. If it is a SELECT query the
-%%              result set is returned, otherwise the number of affected 
-%%       	rows are returned.
 %%--------------------------------------------------------------------------
 sql_query(ConnectionReference, SQLQuery) ->
     sql_query(ConnectionReference, SQLQuery, ?DEFAULT_TIMEOUT).
 
+%%--------------------------------------------------------------------------
+-doc """
+Executes a SQL query or a batch of SQL queries. If it is a SELECT query the
+result set is returned, on the format `{selected, ColNames, Rows}`. For other
+query types the tuple `{updated, NRows}` is returned, and for batched queries,
+if the driver supports them, this function can also return a list of result
+tuples.
+
+> #### Note {: .info }
+>
+> Some drivers may not have the information of the number of affected rows
+> available and then the return value may be `{updated, undefined} `.
+>
+> The list of column names is ordered in the same way as the list of values of a
+> row, e.g. the first `ColName` is associated with the first `Value` in a `Row`.
+""".
+-spec sql_query(ConnectionReference, SQLQuery, TimeOut) -> Result | {error, Reason}  when
+      ConnectionReference :: connection_reference(),
+      SQLQuery :: string(),
+      TimeOut :: erlang:timeout(),
+      Result  :: updated() | selected(),
+      Reason  :: process_not_owner_of_odbc_connection | common_reason().
+%%
+%%--------------------------------------------------------------------------
 sql_query(ConnectionReference, SQLQuery, infinity) when 
   is_pid(ConnectionReference), is_list(SQLQuery) -> 
     ODBCCmd = [?QUERY, SQLQuery],
@@ -194,17 +428,38 @@ sql_query(ConnectionReference, SQLQuery, TimeOut)
     call(ConnectionReference, {sql_query, ODBCCmd}, TimeOut).
 
 %%--------------------------------------------------------------------------
-%% select_count(ConnectionReference, SQLQuery, <TimeOut>) -> {ok, NrRows} |
-%%							    {error, Reason} 
+-doc(#{equiv => select_count/3}).
+-spec select_count(ConnectionReference, SQLQuery) -> {ok, NrRows} |
+          {error, Reason} when
+      ConnectionReference :: connection_reference(),
+      SQLQuery :: string(),
+      NrRows :: n_rows(),
+      Reason :: process_not_owner_of_odbc_connection | common_reason().
 %%                                    
-%% Description: Executes a SQL SELECT query and associates the result set
-%%              with the connection. A cursor is positioned before
-%%        	the first row in the result set and the number of
-%%	        rows in the result set is returned.
 %%--------------------------------------------------------------------------
 select_count(ConnectionReference, SQLQuery) ->	
     select_count(ConnectionReference, SQLQuery, ?DEFAULT_TIMEOUT).
 
+%%--------------------------------------------------------------------------
+-doc """
+Executes a SQL SELECT query and associates the result set with the connection. A
+cursor is positioned before the first row in the result set and the tuple
+`{ok, NrRows}` is returned.
+
+> #### Note {: .info }
+>
+> Some drivers may not have the information of the number of rows in the result
+> set, then `NrRows` will have the value `undefined`.
+""".
+-spec select_count(ConnectionReference, SQLQuery, TimeOut) -> {ok, NrRows} |
+          {error, Reason} when
+      ConnectionReference :: connection_reference(),
+      SQLQuery :: string(),
+      TimeOut :: erlang:timeout(),
+      NrRows :: n_rows(),
+      Reason :: process_not_owner_of_odbc_connection | common_reason().
+%%
+%%--------------------------------------------------------------------------
 select_count(ConnectionReference, SQLQuery, infinity) when 
   is_pid(ConnectionReference), is_list(SQLQuery) ->
     ODBCCmd = [?SELECT_COUNT, SQLQuery],
@@ -216,15 +471,27 @@ select_count(ConnectionReference, SQLQuery, TimeOut) when
     call(ConnectionReference, {select_count, ODBCCmd}, TimeOut).
 
 %%--------------------------------------------------------------------------
-%% first(ConnectionReference, <TimeOut>) ->  {selected, ColNames, Rows} | 
-%%					     {error, Reason} 
-%%                                    
-%% Description: Selects the first row in the current result set. The cursor
-%%            : is positioned at this row. 
+-doc(#{equiv => first/2}).
+-spec first(ConnectionReference) ->  Result | {error, Reason}  when
+      ConnectionReference :: connection_reference(),
+      Result :: selected(),
+      Reason :: result_set_does_not_exist | driver_does_not_support_function |
+                scrollable_cursors_disabled | process_not_owner_of_odbc_connection |
+                common_reason().
 %%--------------------------------------------------------------------------
 first(ConnectionReference) ->	
     first(ConnectionReference, ?DEFAULT_TIMEOUT).	
 
+%%--------------------------------------------------------------------------
+-doc "Returns the first row of the result set and positions a cursor at this row.".
+-spec first(ConnectionReference, TimeOut) -> Result | {error, Reason} when
+      ConnectionReference :: connection_reference(),
+      TimeOut :: erlang:timeout(),
+      Result :: selected(),
+      Reason :: result_set_does_not_exist | driver_does_not_support_function |
+                scrollable_cursors_disabled | process_not_owner_of_odbc_connection |
+                common_reason().
+%%--------------------------------------------------------------------------
 first(ConnectionReference, infinity) when is_pid(ConnectionReference) ->	
     ODBCCmd = [?SELECT, ?SELECT_FIRST],
     call(ConnectionReference, {select_cmd, absolute, ODBCCmd}, infinity);
@@ -235,15 +502,27 @@ first(ConnectionReference, TimeOut)
     call(ConnectionReference, {select_cmd, absolute, ODBCCmd}, TimeOut).
 
 %%--------------------------------------------------------------------------
-%% last(ConnectionReference, <TimeOut>) -> {selected, ColNames, Rows} | 
-%%					   {error, Reason} 
-%%                                    
-%% Description: Selects the last row in the current result set. The cursor
-%%            : is positioned at this row. 
+-doc(#{equiv => last/2}).
+-spec last(ConnectionReference) -> Result | {error, Reason} when
+      ConnectionReference :: connection_reference(),
+      Result :: selected(),
+      Reason :: result_set_does_not_exist | driver_does_not_support_function |
+                scrollable_cursors_disabled | process_not_owner_of_odbc_connection |
+                common_reason().
 %%--------------------------------------------------------------------------
 last(ConnectionReference) ->	
     last(ConnectionReference, ?DEFAULT_TIMEOUT).	
 
+%%--------------------------------------------------------------------------
+-doc "Returns the last row of the result set and positions a cursor at this row.".
+-spec last(ConnectionReference, TimeOut) -> Result | {error, Reason} when
+      ConnectionReference :: connection_reference(),
+      TimeOut :: erlang:timeout(),
+      Result :: selected(),
+      Reason :: result_set_does_not_exist | driver_does_not_support_function |
+                scrollable_cursors_disabled | process_not_owner_of_odbc_connection |
+                common_reason().
+%%--------------------------------------------------------------------------
 last(ConnectionReference, infinity) when is_pid(ConnectionReference) ->	
     ODBCCmd = [?SELECT, ?SELECT_LAST],
     call(ConnectionReference, {select_cmd, absolute, ODBCCmd}, infinity);
@@ -252,17 +531,35 @@ last(ConnectionReference, TimeOut)
   when is_pid(ConnectionReference), is_integer(TimeOut), TimeOut > 0 ->	
     ODBCCmd = [?SELECT, ?SELECT_LAST],
     call(ConnectionReference, {select_cmd, absolute, ODBCCmd}, TimeOut).
+
 %%--------------------------------------------------------------------------
-%% next(ConnectionReference, <TimeOut>) -> {selected, ColNames, Rows} | 
-%%					   {error, Reason}  
-%%                                    
-%% Description: Selects the next row relative the current cursor position 
-%%            : in the current result set. The cursor is positioned at 
-%%            : this row. 
+-doc(#{equiv => next/2}).
+-spec next(ConnectionReference) -> Result | {error, Reason} when
+      ConnectionReference :: connection_reference(),
+      Result :: selected(),
+      Reason :: result_set_does_not_exist | driver_does_not_support_function |
+                scrollable_cursors_disabled | process_not_owner_of_odbc_connection |
+                common_reason().
 %%--------------------------------------------------------------------------
 next(ConnectionReference) ->	
     next(ConnectionReference, ?DEFAULT_TIMEOUT).	
-    
+
+%%--------------------------------------------------------------------------
+-doc """
+Returns the next row of the result set relative the current cursor position and
+positions the cursor at this row. If the cursor is positioned at the last row of
+the result set when this function is called the returned value will be
+`{selected, ColNames,[]}` e.i. the list of row values is empty indicating that
+there is no more data to fetch.[](){: #param_query }
+""".
+-spec next(ConnectionReference, TimeOut) -> Result | {error, Reason} when
+     ConnectionReference :: connection_reference(),
+      TimeOut :: erlang:timeout(),
+      Result :: selected(),
+      Reason :: result_set_does_not_exist | driver_does_not_support_function |
+                scrollable_cursors_disabled | process_not_owner_of_odbc_connection |
+                common_reason().
+%%--------------------------------------------------------------------------
 next(ConnectionReference, infinity) when is_pid(ConnectionReference) ->	
     ODBCCmd = [?SELECT, ?SELECT_NEXT],
     call(ConnectionReference, {select_cmd, next, ODBCCmd}, infinity);
@@ -273,16 +570,30 @@ next(ConnectionReference, TimeOut)
     call(ConnectionReference, {select_cmd, next, ODBCCmd}, TimeOut).
 
 %%--------------------------------------------------------------------------
-%% prev(ConnectionReference, <TimeOut>) -> {selected, ColNames, Rows} | 
-%%					   {error, Reason}   
-%%                                    
-%% Description: Selects the previous row relative the current cursor 
-%%            : position in the current result set. The cursor is
-%%            : positioned at this row. 
+-doc(#{equiv => prev/2}).
+-spec prev(ConnectionReference) -> Result | {error, Reason} when
+      ConnectionReference :: connection_reference(),
+      Result :: selected(),
+      Reason :: result_set_does_not_exist | driver_does_not_support_function |
+                scrollable_cursors_disabled | process_not_owner_of_odbc_connection |
+                common_reason().
 %%--------------------------------------------------------------------------
 prev(ConnectionReference) ->	
     prev(ConnectionReference, ?DEFAULT_TIMEOUT).	
 
+%%--------------------------------------------------------------------------
+-doc """
+Returns the previous row of the result set relative the current cursor position
+and positions the cursor at this row.
+""".
+-spec prev(ConnectionReference, TimeOut) -> Result | {error, Reason} when
+      ConnectionReference :: connection_reference(),
+      TimeOut :: erlang:timeout(),
+      Result :: selected(),
+      Reason :: result_set_does_not_exist | driver_does_not_support_function |
+                scrollable_cursors_disabled | process_not_owner_of_odbc_connection |
+                common_reason().
+%%--------------------------------------------------------------------------
 prev(ConnectionReference, infinity) when is_pid(ConnectionReference) ->	
     ODBCCmd = [?SELECT, ?SELECT_PREV],
     call(ConnectionReference, {select_cmd, relative, ODBCCmd}, infinity);
@@ -293,21 +604,42 @@ prev(ConnectionReference, TimeOut)
     call(ConnectionReference, {select_cmd, relative, ODBCCmd}, TimeOut).
 
 %%--------------------------------------------------------------------------
-%% select(ConnectionReference, <Timeout>) -> {selected, ColNames, Rows} | 
-%%					     {error, Reason}   
-%%                                   
-%% Description: Selects <N> rows. If <Position> is next it is
-%%              semanticly eqvivivalent of calling next/[1,2] <N>
-%%              times. If <Position> is {relative, Pos} <Pos> will be
-%%              used as an offset from the current cursor position to
-%%              determine the first selected row. If <Position> is
-%%              {absolute, Pos}, <Pos> will be the number of the first
-%%              row selected. After this function has returned the
-%%              cursor is positioned at the last selected row.
+-doc(#{equiv => select/4}).
+-spec select(ConnectionReference, Position, N) ->  Result | {error, Reason} when
+      ConnectionReference ::connection_reference(),
+      Position :: next | {relative, integer()} | {absolute, integer()},
+      N :: integer(),
+      Result  :: selected(),
+      Reason :: result_set_does_not_exist | driver_does_not_support_function
+              | scrollable_cursors_disabled | process_not_owner_of_odbc_connection |
+                common_reason().
 %%--------------------------------------------------------------------------
 select(ConnectionReference, Position, N) ->
     select(ConnectionReference, Position, N, ?DEFAULT_TIMEOUT).
 
+%%--------------------------------------------------------------------------
+-doc """
+Selects `N` consecutive rows of the result set. If `Position` is `next` it is
+semantically equivalent of calling `next/[1,2]` `N` times. If `Position` is
+`{relative, Pos}`, `Pos` will be used as an offset from the current cursor
+position to determine the first selected row. If `Position` is
+`{absolute, Pos}`, `Pos` will be the number of the first row selected. After
+this function has returned the cursor is positioned at the last selected row. If
+there is less then `N` rows left of the result set the length of `Rows` will be
+less than `N`. If the first row to select happens to be beyond the last row of
+the result set, the returned value will be `{selected, ColNames,[]}` e.i. the
+list of row values is empty indicating that there is no more data to fetch.
+""".
+-spec select(ConnectionReference, Position, N, TimeOut) ->  Result | {error, Reason} when
+      ConnectionReference ::connection_reference(),
+      Position :: next | {relative, integer()} | {absolute, integer()},
+      N :: integer(),
+      TimeOut :: erlang:timeout(),
+      Result  :: selected(),
+      Reason :: result_set_does_not_exist | driver_does_not_support_function
+              | scrollable_cursors_disabled | process_not_owner_of_odbc_connection |
+                common_reason().
+%%--------------------------------------------------------------------------
 select(ConnectionReference, next, N, infinity) 
   when is_pid(ConnectionReference), is_integer(N), N > 0 ->
     ODBCCmd = [?SELECT, ?SELECT_N_NEXT,
@@ -355,14 +687,46 @@ select(ConnectionReference, {absolute, Pos} , N, TimeOut)
     call(ConnectionReference, {select_cmd, absolute, ODBCCmd},
 	 TimeOut).
 %%--------------------------------------------------------------------------
-%% param_query(ConnectionReference, SQLQuery, Params, <TimeOut>) -> 
-%%                             ok | {error, Reason} 
-%%                                    
-%% Description: Executes a parameterized update/delete/insert-query. 
+-doc(#{equiv => param_query/4}).
+-spec param_query(ConnectionReference, SQLQuery, Params) ->
+          Result | {error, Reason} when
+      ConnectionReference :: connection_reference(),
+      SQLQuery :: string(),
+      Params   :: [{odbc_data_type(), [value()]}] |[{odbc_data_type(), in | out| inout, [value()]}],
+      Result ::  selected() | updated(),
+      Reason :: driver_does_not_support_function |
+                process_not_owner_of_odbc_connection |
+                common_reason().
 %%--------------------------------------------------------------------------
 param_query(ConnectionReference, SQLQuery, Params) ->
     param_query(ConnectionReference, SQLQuery, Params, ?DEFAULT_TIMEOUT).
 
+%%--------------------------------------------------------------------------
+-doc """
+Executes a parameterized SQL query. For an example see the
+["Using the Erlang API"](getting_started.md#param_query) in the Erlang ODBC
+User's Guide.
+
+> #### Note {: .info }
+>
+> Use the function describe_table/\[2,3] to find out which ODBC data type that
+> is expected for each column of that table. If a column has a data type that is
+> described with capital letters, alas it is not currently supported by the
+> param_query function. To learn which Erlang data type corresponds to an ODBC
+> data type see the Erlang to ODBC data type [mapping](databases.md#type) in the
+> User's Guide.
+""".
+-spec param_query(ConnectionReference, SQLQuery, Params, TimeOut) ->
+          Result | {error, Reason} when
+      ConnectionReference :: connection_reference(),
+      SQLQuery :: string(),
+      Params   :: [{odbc_data_type(), [value()]}] |[{odbc_data_type(), in | out| inout, [value()]}],
+      TimeOut :: erlang:timeout(),
+      Result ::  selected() | updated(),
+      Reason :: driver_does_not_support_function |
+                process_not_owner_of_odbc_connection |
+                common_reason().
+%%--------------------------------------------------------------------------
 param_query(ConnectionReference, SQLQuery, Params, infinity) 
   when is_pid(ConnectionReference), is_list(SQLQuery), is_list(Params) ->
     Values = param_values(Params),
@@ -383,17 +747,32 @@ param_query(ConnectionReference, SQLQuery, Params, TimeOut)
     call(ConnectionReference, {param_query, ODBCCmd}, TimeOut).
 
 %%--------------------------------------------------------------------------
-%% describe_table(ConnectionReference, Table, <TimeOut>) -> {ok, Desc} 
-%%
-%% Desc - [{ColName, Datatype}]
-%% ColName - atom()
-%% Datatype - atom()                                    
-%% Description: Queries the database to find out the datatypes of the
-%%              table <Table>
+-doc(#{equiv => describe_table/3}).
+-spec describe_table(ConnectionReference, Table) ->
+          {ok, Description} | {error, Reason} when
+      ConnectionReference :: connection_reference(),
+      Table :: string(),
+      Description :: [{col_name(), odbc_data_type()}],
+      Reason ::  process_not_owner_of_odbc_connection | common_reason().
 %%--------------------------------------------------------------------------
+
 describe_table(ConnectionReference, Table) ->
     describe_table(ConnectionReference, Table, ?DEFAULT_TIMEOUT).
 
+%%--------------------------------------------------------------------------
+-doc """
+Queries the database to find out the ODBC data types of the columns of the table
+`Table`.
+""".
+-spec describe_table(ConnectionReference, Table, TimeOut) ->
+          {ok, Description} | {error, Reason} when
+      ConnectionReference :: connection_reference(),
+      Table :: string(),
+      TimeOut :: erlang:timeout(),
+      Description :: [{col_name(), odbc_data_type()}],
+      Reason ::  process_not_owner_of_odbc_connection | common_reason().
+%%--------------------------------------------------------------------------
+
 describe_table(ConnectionReference, Table, infinity) when 
   is_pid(ConnectionReference), is_list(Table) -> 
     ODBCCmd = [?DESCRIBE, "SELECT * FROM " ++ Table],
@@ -413,6 +792,7 @@ describe_table(ConnectionReference, Table, TimeOut)
 %%            : when connect/2 calls supervisor:start_child/2 to start an 
 %%            : instance of the erlang odbc control process.
 %%--------------------------------------------------------------------------
+-doc false.
 start_link_sup(Args) ->
     gen_server:start_link(?MODULE, Args, []).
 
@@ -428,6 +808,7 @@ start_link_sup(Args) ->
 %%              and starts the port-program that use the odbc driver
 %%		to communicate with the database.
 %%-------------------------------------------------------------------------
+-doc false.
 init(Args) ->
     process_flag(trap_exit, true),
     {value, {client, ClientPid}} = lists:keysearch(client, 1, Args),
@@ -477,6 +858,7 @@ init(Args) ->
 %%              the semantics of result sets.
 %% Note: The order of the function clauses is significant.
 %%--------------------------------------------------------------------------
+-doc false.
 handle_call({Client, Msg, Timeout}, From, State = 
 	    #state{owner = Client, reply_to = undefined})  ->
     handle_msg(Msg, Timeout, State#state{reply_to = From});
@@ -603,7 +985,7 @@ handle_msg({select_cmd, _Type, _ODBCCmd}, _Timeout, State) ->
     {reply, Reply, State#state{reply_to = undefined}};
 
 %---------------------------------------------------------------------------
-%% Catch all -  This can oly happen if the application programmer writes 
+%% Catch all -  This can only happen if the application programmer writes 
 %% really bad code that violates the API.
 handle_msg(Request, _Timeout, State) ->
     {stop, {'API_violation_connection_colsed', Request},
@@ -618,16 +1000,18 @@ handle_msg(Request, _Timeout, State) ->
 %%-------------------------------------------------------------------------
 %% Catch all - This can only happen if the application programmer writes 
 %% really bad code that violates the API.
+-doc false.
 handle_cast(Msg, State) ->
     {stop, {'API_violation_connection_colsed', Msg}, State}.
 
 %%--------------------------------------------------------------------------
 %% handle_info(Msg, State) -> {noreply, State} | {noreply, State, Timeout} |
 %%			      {stop, Reason, State}
-%% Description: Handles timouts, replys from the port-program and EXIT and
+%% Description: Handles timouts, replies from the port-program and EXIT and
 %%		down messages.
 %% Note: The order of the function clauses is significant.
 %%--------------------------------------------------------------------------
+-doc false.
 handle_info({tcp, Socket, BinData}, State = #state{state = connecting, 
 						reply_to = From,
 						odbc_socket = Socket}) ->
@@ -745,7 +1129,7 @@ handle_info({'EXIT', Port, _}, State = #state{erlang_port = Port,
 handle_info({'EXIT', Port, Reason}, State = #state{erlang_port = Port}) ->
     {stop, Reason, State};
 
-%%% If the owning process dies there is no reson to go on
+%%% If the owning process dies there is no reason to go on
 handle_info({'DOWN', _Ref, _Type, _Process, normal}, State) ->
     {stop, normal, State#state{reply_to = undefined}};
     
@@ -775,6 +1159,7 @@ handle_info(Info, State) ->
 %% terminate/2 and code_change/3
 %%--------------------------------------------------------------------------
 
+-doc false.
 terminate({port_exit, _Reason}, State = #state{reply_to = undefined}) ->
     %% Port program crashed
     gen_tcp:close(State#state.odbc_socket),
@@ -795,6 +1180,7 @@ terminate(Reason, State = #state{reply_to = From}) ->
     terminate(Reason, State#state{reply_to = undefined}).
 
 %---------------------------------------------------------------------------
+-doc false.
 code_change(_Vsn, State, _Extra) ->
     {ok, State}.
 
