@@ -31,7 +31,8 @@
 	 test_error/1, test_warning/1, otp_14285/1,
 	 test_if/1,source_name/1,otp_16978/1,otp_16824/1,scan_file/1,file_macro/1,
          deterministic_include/1, nondeterministic_include/1,
-         gh_8268/1
+         gh_8268/1,
+         moduledoc_include/1
         ]).
 
 -export([epp_parse_erl_form/2]).
@@ -76,7 +77,8 @@ all() ->
      encoding, extends, function_macro, test_error, test_warning,
      otp_14285, test_if, source_name, otp_16978, otp_16824, scan_file, file_macro,
      deterministic_include, nondeterministic_include,
-     gh_8268].
+     gh_8268,
+     moduledoc_include].
 
 groups() ->
     [{upcase_mac, [], [upcase_mac_1, upcase_mac_2]},
@@ -130,6 +132,48 @@ file_macro(Config) when is_list(Config) ->
     "Other source" = FileA = FileB,
     ok.
 
+moduledoc_include(Config) when is_list(Config) ->
+    PrivDir = proplists:get_value(priv_dir, Config),
+    ModuleFileContent = <<"-module(moduledoc).
+
+                           -moduledoc {file, \"README.md\"}.
+
+                           -export([]).
+                          ">>,
+    DocFileContent = <<"# README
+
+                        This file is a test
+                       ">>,
+    CreateFile = fun (Dir, File, Content) ->
+                     Dirname = filename:join([PrivDir, Dir]),
+                     ok = create_dir(Dirname),
+                     Filename = filename:join([Dirname, File]),
+                     ok = file:write_file(Filename, Content),
+                     Filename
+                 end,
+
+    %% positive test: checks that all works as expected
+    ModuleName = CreateFile("module_attr", "moduledoc.erl", ModuleFileContent),
+    DocName = CreateFile("module_attr", "README.md", DocFileContent),
+    {ok, List} = epp:parse_file(ModuleName, []),
+    {attribute, _, moduledoc, ModuleDoc} = lists:keyfind(moduledoc, 3, List),
+    ?assertEqual({ok, unicode:characters_to_binary(ModuleDoc)}, file:read_file(DocName)),
+
+    %% negative test: checks that we produce an expected warning
+    ModuleWarnContent = binary:replace(ModuleFileContent, <<"README">>, <<"NotExistingFile">>),
+    ModuleWarnName = CreateFile("module_attr", "moduledoc_err.erl", ModuleWarnContent),
+    {ok, ListWarn} = epp:parse_file(ModuleWarnName, []),
+    {warning,{_,epp,{moduledoc,file, "NotExistingFile.md"}}} = lists:keyfind(warning, 1, ListWarn),
+
+    ok.
+
+create_dir(Dir) ->
+    case file:make_dir(Dir) of
+        ok -> ok;
+        {error, eexist} -> ok;
+        _ -> error
+    end.
+
 deterministic_include(Config) when is_list(Config) ->
     DataDir = proplists:get_value(data_dir, Config),
     File = filename:join(DataDir, "deterministic_include.erl"),
@@ -933,11 +977,12 @@ scan_file(Config) when is_list(Config) ->
     [FileForm1, ModuleForm, ExportForm,
      FileForm2, FileForm3, FunctionForm,
      {eof,_}] = Toks,
-    [{'-',_}, {atom,_,file}, {'(',_} | _ ] = FileForm1,
+    [{'-',_}, {atom,_,file}, {'(',_} | _ ]   = FileForm1,
     [{'-',_}, {atom,_,module}, {'(',_} | _ ] = ModuleForm,
     [{'-',_}, {atom,_,export}, {'(',_} | _ ] = ExportForm,
-    [{'-',_}, {atom,_,file}, {'(',_} | _ ] = FileForm2,
-    [{'-',_}, {atom,_,file}, {'(',_} | _ ] = FileForm3,
+    [{'-',_}, {atom,_,file}, {'(',_} | _ ]   = FileForm2,
+    [{'-',_}, {atom,_,file}, {'(',_} | _ ]   = FileForm3,
+    [{atom,_,ok}, {'(',_} | _]               = FunctionForm,
     ok.
 
 macs(Epp) ->
@@ -1257,8 +1302,15 @@ test_if(Config) ->
 	  {if_8c,
 	   <<"-if(?foo).\n"                     %Undefined symbol.
 	     "-endif.\n">>,
-	   {errors,[{{1,25},epp,{undefined,foo,none}}],[]}}
+	   {errors,[{{1,25},epp,{undefined,foo,none}}],[]}},
 
+	  {if_9c,
+	   <<"-if(not_builtin()).\n"
+	     "a bug.\n"
+	     "-else.\n"
+	     "t() -> ok.\n"
+	     "-endif.\n">>,
+	   {errors,[{{1,21},epp,{bad,'if'}}],[]}}
 	 ],
     [] = compile(Config, Cs),
 
@@ -1324,14 +1376,6 @@ test_if(Config) ->
            ok},
 
 	  {if_7,
-	   <<"-if(not_builtin()).\n"
-	     "a bug.\n"
-	     "-else.\n"
-	     "t() -> ok.\n"
-	     "-endif.\n">>,
-           ok},
-
-	  {if_8,
 	   <<"-if(42).\n"			%Not boolean.
 	     "a bug.\n"
 	     "-else.\n"
@@ -2077,6 +2121,7 @@ gh_8268(Config) ->
     [] = run(Config, Ts),
     ok.
 
+
 %% Start location is 1.
 check(Config, Tests) ->
     eval_tests(Config, fun check_test/3, Tests).
