@@ -1,7 +1,7 @@
 %%
 %% %CopyrightBegin%
 %% 
-%% Copyright Ericsson AB 2006-2021. All Rights Reserved.
+%% Copyright Ericsson AB 2006-2024. All Rights Reserved.
 %% 
 %% Licensed under the Apache License, Version 2.0 (the "License");
 %% you may not use this file except in compliance with the License.
@@ -84,8 +84,7 @@ all() ->
      resend_server,
      large_file,
      app,
-     appup,
-     start_tftpd
+     appup
     ].
 
 groups() -> 
@@ -115,36 +114,6 @@ appup() ->
 appup(Config) when is_list(Config) ->
     ok = test_server:appup_test(tftp).
 
-start_tftpd() ->
-    [{doc, "Start/stop of tfpd service"}].
-start_tftpd(Config) when is_list(Config) ->
-    process_flag(trap_exit, true),
-    ok = tftp:start(),
-    {ok, Pid0} = tftp:start_service([{host, "localhost"}, {port, 0}]),
-    Pids0 =  [ServicePid || {_, ServicePid} <- tftp:services()],
-    true = lists:member(Pid0, Pids0),
-    {ok, [_|_]} = tftp:service_info(Pid0),
-    tftp:stop_service(Pid0),
-    ct:sleep(100),
-    Pids1 =  [ServicePid || {_, ServicePid} <- tftp:services()],
-    false = lists:member(Pid0, Pids1),
-
-    {ok, Pid1} =
-	tftp:start_standalone([{host, "localhost"}, {port, 0}]),
-    Pids2 =  [ServicePid || {_, ServicePid} <- tftp:services()],
-    false = lists:member(Pid1, Pids2),
-    %% Standalone service is not supervised
-    {error,not_found} = tftp:stop_service(Pid1),
-    ok = tftp:stop(),
-
-    application:load(tftp),
-    application:set_env(tftp, services, [{tftpd, [{host, "localhost"},
-                                                  {port, 0}]}]),
-    ok = tftp:start(),
-    1 = length(tftp:services()),
-    application:unset_env(tftp, services),
-    ok = tftp:stop().
-
 
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %% Simple
